/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/12/15 18:49:11 $
 */
package kaos.core.csi;

import kaos.core.csi.registration.Registration;
import kaos.core.csi.registration.ModificationResult;
import kaos.core.csi.registration.ModificationResultImpl;
import kaos.core.csi.registration.Property;
import kaos.core.csi.registration.PropertyImpl;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.service.directory.KAoSEntityDescription;
import kaos.core.service.directory.DefaultKAoSAgentDescription;
import kaos.core.service.directory.ModificationException;
import kaos.core.util.Logger;
import kaos.core.util.MsgUtils;
import kaos.core.util.KAoSConstants;

import javax.agent.Locator;
import javax.agent.service.transport.NoSuchTransportException;
import javax.agent.TransportMessage;
import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.directory.NotRegisteredException;
import javax.agent.service.transport.MessageListener;
import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;

import ri.acr.AclMessage;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import java.io.Serializable;


/**
 * $Revision: 1.23 $
 */
public class KAoSActorImpl
    implements  MessageListener, KAoSActor
   {

    
    private static final String         PROPERTY_UPDATE_KEY = "PROPERTY_UPDATE_KEY";
    
    //KAoSActor vars
    protected KAoSAgentDescription      _actorDesc; 
    protected Registration              _registration;
    protected TransportSupport          _transportSupport;
    protected boolean                   _autoCommit = true;
    protected boolean                   _registered = false;
    protected String                    _preferredTransportName = null;
    protected Hashtable                 _transports;
    protected List                      _propertiesToModify = null;
    private Hashtable                   _updateLock = new Hashtable(1);
    protected  Logger             	    _logger = Logger.getLogger(this);
    //JAS
    protected Locator                   _lctr;
    
   public KAoSActorImpl()
   {
	   
   }
    /**
     * Instantiates a new KAoS-aware agent using JAS services for transport, naming and 
     * agent directory.
     */
    public KAoSActorImpl(String guid,
                         String name,
                         Vector domainNames,
                         Transport transport)
    throws Exception
    {
    	initialize(guid,
 			       name,
 			       domainNames,
 			       transport);
    }

   /**
    * Instantiates a new KAoS-aware agent using JAS services for transport, naming and 
    * agent directory.
    */
   public KAoSActorImpl(String [] args)
   throws Exception
   {
       String name = null;
       String transport = null;
       Vector domainNames = null; 
       if (args.length > 0) {
	        int i = 0;
	        while (i < args.length) {
	            if (args[i].equalsIgnoreCase ("-name")) {
	                if ((i+1) < args.length) {
	                	name = args[i+1];
	                	i++;
	                }
	                else {
	                	System.out.println ("\n	**** Must specify Agent Name with -name");
	                	throw new Exception();
	                }
	            }
	            else if (args[i].equalsIgnoreCase ("-transport")) {
	                if ((i+1) < args.length) {
	                	transport = args[i+1];
	                	i++;
	                }
	                else {
	                	System.out.println ("\n	**** Must specify Agent transport with -transport");
	                	throw new Exception();
	                }
	            }
	            else if (args[i].equalsIgnoreCase ("-domains")) {
                    if ((i+1) < args.length) {
                        String domainsStr = args[i+1];
                        domainNames = new Vector();
                        StringTokenizer tokenizer = new StringTokenizer(domainsStr, ",");
                        while (tokenizer.hasMoreTokens()) {
                            domainNames.add(tokenizer.nextToken());
                        }
                    }
                }// else if
	            i++;
	        }
	    }
       
       	if ( domainNames == null ) {
    	   domainNames = new Vector();
    	   // try the following property, if not specified on the command line
    	   String domainNamesStr = System.getProperty(KAoSConstants.DOMAIN_NAMES);
    	   if ( domainNamesStr != null ) {  		 
    		   StringTokenizer st = new StringTokenizer (domainNamesStr, ",");
    		   while (st.hasMoreTokens()) {
    			   String str = st.nextToken();
    			   domainNames.addElement (str);
    		   }
    	   }
       } 
       
       if (domainNames.isEmpty()) {
    	   domainNames.add("TestDomain");
           System.out.println (" <>    Warning: No Domain Names specified, using default: " + domainNames);
       }
	   
	    Transport trans = new TransportImpl();
	    trans.setName(transport);
      
	    initialize(null,
			       name,
			       domainNames,
			       trans);
   }
    /**
     * 
     * @param guid
     * @param name
     * @param domainNames
     * @param transport
     * @throws Exception
     */
    public void initialize(String guid,
            			   String name,
            			   Vector domainNames,
            			   Transport transport)
    throws Exception
    {    
    	_transportSupport = CSIFactory.getTransportSupport();
    	
    	String transportName = null;
    	if ( transport != null ) {
    		transportName = transport.getName();
            if ( transportName != null ) {
                if ( ! _transportSupport.isTransportSupported(transportName) )
                throw new Exception("Reguested transport: " + transportName + " is not supported! \n  Supported transports: " 
                                     + _transportSupport.getSupportedTransportsNames() );
                _preferredTransportName = transportName;   
            }
        }
    	
    	System.out.println ("\n     <>  Agent name:                 " + name);
 	    System.out.println ("\n     <>  Requested transport:        " + transportName);
 	    System.out.println ("\n     <>  Requested domains:          " + domainNames + "\n");
 	    
        String dsTransportName = System.getProperty(KAoSConstants.DIRECTORY_TRANSPORT);
        if ( dsTransportName != null ) {
        	CSIFactory.setCurrentTransport(dsTransportName);
            _registration = CSIFactory.getRegistration();   
        }
        else{
            CSIFactory.setCurrentTransport(_preferredTransportName);
            _registration = CSIFactory.getRegistration();   
        }
        
        // Try to register domains.
        Enumeration domainsEnum = domainNames.elements();
        while ( domainsEnum.hasMoreElements() ) {
            String domainName  = (String) domainsEnum.nextElement();
            System.out.println("\n     <>  Will register domain: " + domainName + "\n");
            
            try {
            	_registration.registerDomain(domainName);
            }
            catch(DirectoryFailure de) {
                System.out.println("\n   Due to Directory Failure unable to register domain: " + domainName);
                de.printStackTrace();
                throw de;
            }
            catch(AlreadyRegisteredException ae) {
                System.out.println("    The following domain has already been registered: " + domainName);
            }
        }
          
        // Obtain the KAoSAgentDescription
        _actorDesc = _registration.createKAoSAgentDescription(guid,
                                                              name,
                                                              domainNames, 
                                                              _preferredTransportName,
                                                              null);
        
        // Ask TransportSupport to bind this MessageListener to the given transport, so the receiver
    	// will be able to receive messages.
        _transportSupport.bindMsgListenerToTransport(this, name, _actorDesc);
        
        _transports = new Hashtable();
        _transports.put(transport.getName(),transport);
     
        
       
    }
    //////////////////////////////////////////////////////////////
    // Implementation of the KAoSActor interface
   
    /** 
     * Get the name of the concept mapping between the entity system's ontological concepts and KAoS ontology. 
     *
     * @return       String specifying the name of the mapping to be used to translate ontological names from
     *               the entity system to KAoS ontologies.  The default name is "KAoS" indicating KAoS ontologies
     *               to be used, no translation necessary.
     */
    public String getConceptMappingName()
    {
        return _actorDesc.getConceptMappingName();
    }
   
    /** 
     * Set the name of the concept mapping between the entity system's ontological concepts and KAoS ontology. 
     *
     * @param mappingName    The name of the mapping to be used to translate ontological names from
     *                       the entity system to KAoS ontologies.
     */
    public void setConceptMappingName(String mappingName)
    {
        _actorDesc.setConceptMappingName(mappingName);  
    }
    
    /** 
     * Returns a List of String containing the names of domains the actor is registered in.  
     * Note: When the actor first registers with the DS, this List is checked and, if not empty, the actor 
     * will be registered in the specified domains.  Subsequently, if the actor wants to register 
     * in additional domains, then the actor has to lookup the desired domain and register in it 
     * via the Domain.registerAgent() method, which will update this List after successful registration.
     *
     * @return       List of domain names contained by this KAoSActor.
     */
    public List getDomainNames()
    {
        return _actorDesc.getDomainNames();
    }

    /**
     * Obtain the actor's globally unique identifier.
     *
     * @return String representing the actor's guid.
     */
    public String getGUID()
    {
        return _actorDesc.getEntityNameAsString();
    }
    
    /**
     * Set the actor's globally unique identifier.
     *
     * @param actorGUID   String containing actor's globally unique identifier.
     */
    public void setGUID(String actorGUID)
    {
        _actorDesc.setEntityNameAsString(actorGUID);
    }
    
    /**
     * Obtain the actor's name.
     *
     * @return String representing the actor's name.
     */
    public String getName()
    {
        return _actorDesc.getAgentNickname();
    }
    
    /**
     * Set the actor's name.
     *
     * @param name   String containing the actor's name.
     */
    public void setName(String name)
    {
        _actorDesc.setAgentNickname(name);
    }
    
    
     /** 
     * Add an ontological type to the collection of types held by the KAoSActor.
     * The default for an agent is http://ontology.ihmc.us/Actor.owl#Agent.
     *
     * @param type       String containing the name of ontological type of this actor.
     *
     * @param owlDesc    String containing OWL description of the corresponding ontological type.
     */
    public void addOntologicalType(String type, String owlDesc)
    {
        if ( ! _registered ) {
            _actorDesc.addEntityOntologicalType(type, owlDesc);
        }
        else {
            Hashtable newType = new Hashtable(1);
            newType.put(type,owlDesc);
            ModificationResultImpl modResult = new ModificationResultImpl();
            modResult.setModificationType(ModificationResult.ADD_ELEMENT_TO_TABLE);
            PropertyImpl property = new PropertyImpl(KAoSEntityDescription.ENTITY_ONTOLOGICAL_TYPES,
                                                     "",
                                                     modResult,
                                                     0,
                                                     newType);
            try {
                updatePropertyValue(property);
            }
            catch(ModificationException e) {
                e.printStackTrace();
            }
        }
        
    }
   
    /** 
     * Remove an ontological type from the collection of types held by the KAoSActor. 
     *
     * @param type   The name of the ontological type to be removed from the collection of ontological types.
     *
     * @return       <tt>true</tt> if the collection of ontological types contained the specified type; otherwise return <tt>false</tt>.
     */
    public boolean removeOntologicalType(String type)
    {
        if ( ! _actorDesc.containsKey(type) ) return false;
        if ( ! _registered ) {
            _actorDesc.removeEntityOntologicalType(type);
            return true;
        }
        else {
            ModificationResultImpl modResult = new ModificationResultImpl();
            modResult.setModificationType(ModificationResult.DELETE_ELEMENT_FROM_TABLE);
            PropertyImpl property = new PropertyImpl(KAoSEntityDescription.ENTITY_ONTOLOGICAL_TYPES,
                                                     "",
                                                     modResult,
                                                     0,
                                                     type);
            try {
                updatePropertyValue(property);
            }
            catch(ModificationException e) {
                e.printStackTrace();
                return false;
            }
            return true;
        }
    }
   
    /** 
     * Set the ontological types of this KAoSActor. 
     * NOTE:  The new types replace existing ones, if there were any set!
     *
     * @param types   Map containing type name -> OWL description mapping of ontological types
     *                of the entity this description is associated with. 
     */
    public void setOntologicalTypes(Map types)
    {
        if ( ! _registered ) {
            _actorDesc.setEntityOntologicalTypes(types);
        }
        else {
            ModificationResultImpl modResult = new ModificationResultImpl();
            modResult.setModificationType(ModificationResult.REPLACE);
            PropertyImpl property = new PropertyImpl(KAoSEntityDescription.ENTITY_ONTOLOGICAL_TYPES,
                                                     "",
                                                     modResult,
                                                     0,
                                                     (Serializable)types);
            try {
                updatePropertyValue(property);
            }
            catch(ModificationException e) {
                e.printStackTrace();
            }
            
        }
    }
    
    /**
     * Obtain the collection of ontological types of this KAoSActor. 
     *
     * Presently, the types are defined by constants in kaos.ontology.vocabulary.ActorConcepts.  
     *
     * @return Map containing mappings of ontological type names -> OWL descriptions of the types 
     *         or null, if ontological types have not been set.
     */
    public Map getEntityOntologicalTypes()
    {
        return _actorDesc.getEntityOntologicalTypes();
    }
    
    /**
     * Obtain the collection of just names of ontological types of KAoS entity described by this KAoSActor. 
     *
     * Presently, the types are defined by constants in kaos.ontology.jena.ActorConcepts.  
     *
     * @return Enumeration containing names of ontological types or null, if there no types specified in the KAoSActor.
     */
    public Enumeration getEntityOntologicalTypeNames()
    {
        return _actorDesc.getEntityOntologicalTypeNames();
        
    }
    
    /**
     * Obtain guarded/unguarded status of this KAoSActor.
     *
     * @return boolean indicating whether the KAoSActor is guarded. 
     */
    public boolean isGuarded()
    {
        List guardIDs = _actorDesc.getGuardIDs();
        return ( guardIDs.size() > 0 );
    }
   
    /**
     * Obtain registered/unregistered status of this KAoSActor.
     *
     * @return boolean indicating whether the KAoSActor has been registered in the KAoS DS. 
     */
    public boolean isRegistered()
    {
        return _registered;
    }
    
    /**
     * Set the  registered/unregistered status of this KAoSActor.
     *
     * @param  registered boolean indicating whether the KAoSActor has been registered in the KAoS DS. 
     */
    public void setRegistered(boolean registered)
    {
        _registered = registered;
        logMessage(".setRegistered: " + _actorDesc.getEntityNameAsString() + " registration status has been set to: "
                   + registered + ".", Logger.INFO);
    }
    
    /** 
     * Add an AgentCapability this KAoSActor offers.
     *
     * @param capability    AgentCapability describing individual capability of this KAoSActor.
     */
    public void addCapability(AgentCapability capability)
        throws ModificationException
    {
        if ( ! _registered ) {
            _actorDesc.addCapability(capability);
        }
        else {
            ModificationResultImpl modResult = new ModificationResultImpl();
            modResult.setModificationType(ModificationResult.ADD_ELEMENT_TO_LIST);
            PropertyImpl property = new PropertyImpl(KAoSAgentDescription.AGENT_CAPABILITIES,
                                                     "",
                                                     modResult,
                                                     0,
                                                     capability);
            updatePropertyValue(property);
        }//end else if registered
    }
   
    /** 
     * Remove the specified AgentCapability from the collection of this KAoSActor's capabilities. 
     *
     * @param capability   The AgentCapability to be removed from the collection of this KAoSActor's capabilities.
     *
     * @return       <tt>true</tt> if the collection of capabilities contained the specified capability; otherwise return <tt>false</tt>.
     */
    public boolean removeCapability(AgentCapability capability)
        throws ModificationException
    {
        if ( ! _actorDesc.containsKey(capability.getName()) ) return false;
        if ( ! _registered ) {
            _actorDesc.removeCapability(capability);
        }
        else {
            ModificationResultImpl modResult = new ModificationResultImpl();
            modResult.setModificationType(ModificationResult.DELETE_ELEMENT_FROM_LIST);
            PropertyImpl property = new PropertyImpl(KAoSAgentDescription.AGENT_CAPABILITIES,
                                                     "",
                                                     modResult,
                                                     0,
                                                     capability);
            updatePropertyValue(property);
        }
        return true;
    }
   
    /** 
     * Set the collection of AgentCapabilities of this KAoSActor.  
     *
     * @param capabilities   List containing the AgentCapabilities of this KAoSActor.  
     */
    public void setCapabilities(List capabilities)
        throws ModificationException
    {
        if ( ! _registered ) {
            _actorDesc.setCapabilities(capabilities);
        }
        else {
            if ( ! (capabilities instanceof Serializable) ) 
                throw new ModificationException("Capabilities not Serializable, unable to update in the DS! They are: "
                                                + capabilities.getClass().getName());
            Serializable  capabilitiesSer = (Serializable) capabilities;
            ModificationResultImpl modResult = new ModificationResultImpl();
            modResult.setModificationType(ModificationResult.REPLACE);
            PropertyImpl property = new PropertyImpl(KAoSAgentDescription.AGENT_CAPABILITIES,
                                                     "",
                                                     modResult,
                                                     0,
                                                     capabilitiesSer);
            updatePropertyValue(property);
        }
    }
    
    /**
     * Obtain the collection of AgentCapabilities of this KAoSActor. 
     *
     * @return List of AgentCapability describing the capabilities  of the actor.
     */
    public List getCapabilities()
    {
        return _actorDesc.getCapabilities();
    }
    
    ////    ******  GENERIC PROPERTY MANAGMENT API 
    
    /** 
     * Set a Hashtable containing a mapping of actor properties.  These properties 
     * will be stored as part of the actor registration info in the Directory Service
     * under the specified keys.
     *
     * @param properties   Hashtable containing properties this KAoSActor.  
     */
    public void setProperties(Hashtable properties)
    {
        Enumeration myEnum = properties.keys();
        while ( myEnum.hasMoreElements() ) {
            String key = (String)myEnum.nextElement();
            Object value = properties.get(key);
            if ( value instanceof Property ) {
                Property prop = (Property) value;
                _actorDesc.set(key,prop.getValue());
            }
            else{
                _actorDesc.set(key,value);    
            }
        }

    }
    
    /**
     * Returns a Hashtable containing a mapping of actor properties.  
     * These properties will be stored as part of the actor registration info 
     * in the Directory Service under the specified keys.  After registration 
     * properties can be modified via methods add/remove/setProperty methods.
     *
     * @return Hashtable of actor's properties.
     */
    public Hashtable getProperties ()
    {
        Hashtable properties = new Hashtable();
        Enumeration myEnum = _actorDesc.keys();
        while ( myEnum.hasMoreElements() ) {
            String key = (String)myEnum.nextElement();
            Object value = _actorDesc.get(key);
            if ( (value instanceof Serializable) ) {
                Property prop = new PropertyImpl();
                prop.setName(key);
                prop.setValue((Serializable)value);
                properties.put(key,prop);
            }
        }
        return properties;
    }
    
     /**
     * Indicates whether property value changes are automatically committed to the Directory Service 
     * (default = true). Calls to addPropertyValue, removePropertyValue and setPropertyValue will,
     * when true, result in the immediate update of these value changes in the Directory Service. 
     * When false these value changes will not occur in the Directory Service until commitPropertyValues is called.
     *
     * @return booelan setting the auto commit flag.
     */
    public boolean isAutoPropertyValueCommit()
    {
        return _autoCommit;
    }
    
    /**
     * Set the auto property value commit flag to true or false.
     *
     * @param autoCommit    boolean specifying the auto commit flag.
     *
     */
    public void setAutoPropertyValueCommit(boolean autoCommit)
    {
        _autoCommit = autoCommit;
    }
    
    /**
     * Add the specified property, whose name and value are given to the collection of properties of this KAoSActor.
     *
     * @param property      Property to be added.
     *
     * @param value         Object specifying the property value.
     */
    public void addProperty(Property property, Object value)
    {
        try {
            processPropertyUpdate(ModificationResult.ADD, property, value);
        }
        catch(ModificationException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Remove the specified property from the collection of properties of this KAoSActor.
     *
     * @param property      Property to be removed.
     *
     */
    public void removeProperty(Property property)
    {
        try {
            processPropertyUpdate(ModificationResult.DELETE, property, null);
        }
        catch(ModificationException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Commit any property value changes that were not yet send to the Directory Service. 
     * The use of this method allows the developer to make multiple property changes before 
     * sending these changes to the Directory Service to allow for a more optimized use of network bandwidth
     *
     * @throws  ModificationException when one or more property changes failed to be committed to the Directory Service.
     *          The exception lists the properties for which the modification failed.
     *
     */
    public void commitPropertyValues()
        throws ModificationException
    {
        if ( ( _propertiesToModify == null) || ( _propertiesToModify.isEmpty() ))  return;
        
        synchronized(_updateLock) {
            while ( _updateLock.containsKey(PROPERTY_UPDATE_KEY) ) {
                try {
                    _updateLock.wait();
                }
                catch (InterruptedException xcp) {
                     logMessage(".commitPropertyValues: while waiting to update properties, waiting thread has been interrupted!! ", xcp, Logger.ERROR);
                     return;
                }
            }
            
            if ( ( _propertiesToModify == null) || ( _propertiesToModify.isEmpty() ))  return;
            _updateLock.put(PROPERTY_UPDATE_KEY,PROPERTY_UPDATE_KEY);
        }
        
        ModificationException expToThrow = null;
        try {
            _registration.modifyEntityProperties(_actorDesc.getEntityNameAsString(),_propertiesToModify);
            updateLocalActorDescription(_propertiesToModify);
        }
        catch(Exception e){
            e.printStackTrace();
            if (e instanceof ModificationException) {
                ModificationException modException = (ModificationException) e;
                List failures = modException.getFailedMods();
                if ( ! ( (failures == null)  ||  (failures.isEmpty()) ) ) {
                    for (Iterator failuresIt = failures.iterator(); failuresIt.hasNext();) {
                        Property prop = (Property) failuresIt.next();
                        ModificationResult resultEX = prop.getModificationResult();
                        logMessage("\n Modification succeded: " + resultEX.isSuccess() 
                                       + "\n      Failure description: " + resultEX.getModificationFailureDescription(), Logger.INFO);
                    }
                
                    List validProps = getValidPropertyMods(failures);
                    updateLocalActorDescription(validProps);
                }
                expToThrow = modException;
            }
            if (e instanceof  DirectoryFailure) {
                logMessage("\n\n   Due to Directory Failure the following properties have not been updated: " 
                            + _propertiesToModify, Logger.ERROR);
                expToThrow = new ModificationException("\n\n   Due to Directory Failure the included properties have not been updated!!!" );
                expToThrow.setFailedMods(copyProps(_propertiesToModify));
                
            }
            if (e instanceof  NotRegisteredException) {
                logMessage("\n\n   Actor: " + _actorDesc.getEntityNameAsString() + " has not been registered in the DS. The following properties have not been updated: " 
                            + _propertiesToModify, Logger.ERROR);
                expToThrow = new ModificationException("\n\n   Actor: " + _actorDesc.getEntityNameAsString() 
                                                      + " has not been registered in the DS. The included properties have not been updated!!!" );
                expToThrow.setFailedMods(copyProps(_propertiesToModify));
            }
        }// end catch
        
        _propertiesToModify = new ArrayList();
        
        synchronized(_updateLock) {
            _updateLock.remove(PROPERTY_UPDATE_KEY);
            _updateLock.notifyAll();
        } 
        if ( expToThrow != null )  throw expToThrow;
    }
      
    /**
     * Return the property, whose name is specified
     *
     * @param name  String containing the name of the property to be returned.
     */
    public Property getProperty(String name)
    {
        Object value = _actorDesc.get(name);
        if ( (value instanceof Serializable) ) {
            Property prop = new PropertyImpl();
            prop.setName(name);
            prop.setValue((Serializable)value);
            return prop;
        }
        return null;
    }
    
    /**
     * Add the specified value to the list of values of the given property.
     *
     * @param property      Property, whose value is to be added.
     *
     * @param value         Object specifying the new value to be added to property values.
     *
     * @throws  ModificationException when auto commit is true and when an error o
     *           ccurred trying to update the property in the directory service.
     */
    public void addPropertyValue(Property property, Object value)
        throws ModificationException
    {
        processPropertyUpdate(ModificationResult.ADD_ELEMENT_TO_LIST, property, value);
    }
        
    /**
     * Remove the specified value from the list of values of the given property.
     *
     * @param property      Property, whose value is to be removed.
     *
     * @param value         Object specifying the new value to be added to property values.
     *
     * @throws  ModificationException when auto commit is true and when an error occurred 
     *          trying to update the property in the Directory Service.
     */
    public void removePropertyValue(Property property, Object value)
        throws ModificationException
    {
        processPropertyUpdate(ModificationResult.DELETE_ELEMENT_FROM_LIST, property, value);
    }
        
    /**
     * Replace the specified  value with the given value of the given property
     *
     * @param property      Property, whose value is to be replaced.
     *
     * @param value         Object specifying the new value to replace a property value.
     *
     * @throws  ModificationException when auto commit is true and when an error occurred 
     *          trying to update the property in the Directory Service.
     */
    public void setPropertyValue(Property property, Object value)
        throws ModificationException
    {
        processPropertyUpdate(ModificationResult.REPLACE, property, value);
    }
        
    /**
     * Return the value of the given property
     *
     * @param property  Property, whose value is to be returned.
     *
     * @return Object   containing the given Property value.
     */
    public Object getPropertyValue(Property property)
    {
        return _actorDesc.get(property.getName());
    }
    
     ////    ******  TRANSPORT API 
    
    /**
     * Send  the specified  message content to the given KAoSActor.
     *
     * @param messageContent        Serializable containing the message to be sent out.
     *
     * @param receiver         KAoSAgentDescription containing the description of the receiver.
     *     
     * @throws  NotLocatableException when the receiver cannot be located.
     *
     * @throws  TransportFailure when the transport system fails.
     */
     public void sendMessage(Serializable messageContent, KAoSActor receiver) 
        throws NotLocatableException,
               TransportFailure
    {
        KAoSAgentDescription  receiverDesc = null;
        if ( receiver instanceof BasicKAoSActorImpl ) {
           BasicKAoSActorImpl basicReceiver = (BasicKAoSActorImpl)receiver;   
           receiverDesc =  basicReceiver.getAgentDescription();
        }
       else if ( receiver instanceof KAoSActorImpl  ) {
            KAoSActorImpl receiverActor = (KAoSActorImpl) receiver;
            receiverDesc = receiverActor.getAgentDescription();
       }
       
       if ( receiverDesc == null ) {
            throw new NotLocatableException("\n   *** Unable to obtain Locator for the following receiver: " 
                                             + receiver.getName() + " Message: " + messageContent + "\n cannot be sent!");
       }
       
        _transportSupport.sendMessageTo(_actorDesc.getLocators()[0],
                                         receiverDesc.getLocators()[0],
                                        _actorDesc.getAgentNickname(),
                                         receiverDesc.getAgentNickname(),
                                         null,
                                         messageContent,
                                        _preferredTransportName);
                                                
    }
               
    /**
     * Receive  the specified  message content from the given KAoSActor.
     *
     * @param messageContent        Serializable containing the received message. 
     *
     * @param sender                KAoSAgentDescription containing the description of the sender. 
     *
     *
     */
    public void receiveMessage(Serializable messageContent,  KAoSActor sender)
    {
        logMessage(".receiveMessage: \n *** RECEIVED: " + messageContent + "\n  Sender: " + sender.getName(), Logger.INFO);
    }
    
    /**
     * Return the List of valid Transports.
     *
     * @return List     containing the supported Transports.
     */
    public List getTransports()
    {
        ArrayList tranList = new ArrayList();
        Enumeration myEnum = _transports.elements();
        while ( myEnum.hasMoreElements() ) {
            Transport transport = (Transport)myEnum.nextElement();
            tranList.add(transport);
        }
        
        return tranList;
    }
    
    /**
     * Add the specified Transport to the collection of available Transports. 
     *
     * @param transport     Transport to be added to the collection of actor's Transports.
     */
    public void addTransport(Transport transport)
        throws NoSuchTransportException
    {
         if ( ! _transportSupport.isTransportSupported(transport.getName()) )
            throw new NoSuchTransportException();
            
        if ( _transports.containsKey(transport.getName()) ) {
            logMessage(".addTransport: Specified transport: " + transport.getName() + " already exists.  Ignoring request.",
                        Logger.INFO);
            return;
        }
        
        _transports.put(transport.getName(), transport);
    }
    
    /**
     * Remove the specified Transport from the collection of actor's Transports. 
     *
     * @param transport     Transport to be removed from the collection of actor's Transports.
     */
    public void removeTransport(Transport transport)
    {
       _transports.remove(transport.getName()); 
    }
    
    /**
     * Return the Transport, whose name is specified.
     * 
     * @param name          String containing the Transport name to be retrieved.
     *
     * @return Transport    whose name was specified.
     */
    public Transport getTransport(String name)
    {
        return (Transport) _transports.get(name);
    }

    //////////////////////////////////////////////////////////////
    // Implementation of the MessageListener interface

    /**
     * Implements the receiveMessage method mandated by the
     * MessageListener interface.  This method prints
     * the incoming TransportMessage to System.out.
     *
     * @see MessageListener
     * @see TransportMessage
     * @see "JAS Specification, Section(s) 3.4.2, 4.7.2"
     */
    public void receiveMessage( TransportMessage tmsg )
    {
        printTransportMessage( tmsg );
        Object msgContent = MsgUtils.unwrapObjectContent(tmsg);

        if ( msgContent == null ) {
            logMessage(".receiveMessage: \n receiveMessage: could not retrieve the content from: " + tmsg, Logger.INFO);
            return;
        }
                    
        String senderName;
        AclMessage acrAclMsg = MsgUtils.getAclMessage(tmsg);
        if ( acrAclMsg == null ) {
            //msender = new AcrAgentName("UNKNOWN" );
            senderName = "UNKNOWN";
        }
        else {
            senderName = acrAclMsg.getSender().getName();
            
        }
        logMessage(".receiveMessage: received content: " + msgContent + " from: " + senderName, Logger.INFO);
        Locator senderLoc = tmsg.getSender();
        DefaultKAoSAgentDescription desc = new DefaultKAoSAgentDescription();
        desc.removeExecutionEnv();
        desc.removeHostAddress();
        desc.removeHostName();
        desc.addLocator(senderLoc);
        desc.setAgentNickname(senderName);
        KAoSActor senderActor = new BasicKAoSActorImpl(desc);
        receiveMessage((Serializable)msgContent, senderActor);

    }

    /**
     * Two agents are equal if their respective hash codes
     * are equal.
     *
     * @return true if obj equals this KAoSActor.
     *
     * @see #hashCode
     */
    public boolean equals( Object obj )
    {
        if ( obj == this )
            return true;

        if ( !(obj instanceof KAoSActor) )
            return false;

        return this.hashCode() == ((KAoSActor)obj).hashCode();
    }

    /**
     * Determines the hash code for the KAoSActor.  The KAoSActor
     * hash code is simply the hash code of the represented agents
     * AgentName.  If the AgentName field has not yet been set,
     * then the hash code is found from <code>super.hashCode()
     * </code>.
     *
     * @return int the hashCode for this KAoSActor.
     *
     */
    public int hashCode()
    {
        return getGUID() == null ? super.hashCode() : getGUID().hashCode();
    }

    //////////////////////////////////////////////////////////////
    

    ////    ******  OTHER PUBLIC API 
    public KAoSAgentDescription getAgentDescription()
    {
        return _actorDesc;
    }
    
    public void registerWithKAoS()
    throws Exception
    {
    	 _registration.registerEntity(_actorDesc,
                                      true,       // guarded
                                      false);     // replace registration  

    	 setRegistered(true);
    }
    //////////////////////////////////////////////////////////////
    
    
    
    protected void printTransportMessage( TransportMessage tmsg )
    {
        logMessage( "\n   *****   Incoming transport message:\n"+ tmsg, Logger.DEBUG );
    }
      
    protected void logMessage(String msg, int logLevel)
    {
   		if (_logger.isEnabledFor(logLevel))
   			_logger.logMessage(msg, logLevel);
    }
	
    protected void logMessage(String msg, Exception e, int logLevel)
    {
   		if (_logger.isEnabledFor(logLevel))
   			_logger.logMessage(msg, e, logLevel);
    }
    
    protected void setMyDescription(KAoSAgentDescription  desc)
    {
        if ( desc != null ) _actorDesc = desc;
        
    }

    //////////////////////////////////////////////////////////////
    // PRIVATE METHODS
    private void processPropertyUpdate(String updateType, Property property, Object value)
        throws ModificationException
    {
       // set the update type in the ModificationResult of the property.
        ModificationResultImpl modResult = new ModificationResultImpl();
        modResult.setModificationType(updateType);
        property.setModificationResult(modResult);
        if ( ! ( value instanceof Serializable) ) {
            throw new ModificationException("Property value not instance of Serializable: " + value
                                             + " The following property cannot be modified: \n" + property);
        }
        property.setValue((Serializable)value);
        switch(property.getScope()){
            case 0 : { //PUBLIC scope indicates updates to the Directory Service and to the local properties
                updatePropertyValue(property);
            }//case 0
            case 1 : { //PRIVATE scope indicates local updates only
                ArrayList props = new ArrayList(1);
                props.add(property);
                _actorDesc.modifyProperties(props);
                break;
            }//case 1
            default: {
                throw new ModificationException("Unrecognized scope for property modification: " + property.getScope()
                                                + " The following property cannot be modified: \n" + property);
            }
        } 
    }
    
    private void updatePropertyValue(Property property)
        throws ModificationException
    {
        if  ( _propertiesToModify == null )  _propertiesToModify = new ArrayList();
        
        while ( _updateLock.containsKey(PROPERTY_UPDATE_KEY) ) {
            try {
                synchronized(_updateLock) {
                    // this waiting thread will be notified in the commitPropertyValues() method,
                    // after the update in progress has been completed.
                    _updateLock.wait();
                }
            }
            catch (InterruptedException xcp) {
                logMessage(".updatePropertyValue: while waiting to update a property, waiting thread has been inturrupted!!  Property: "
                           + property + "\n    cannot be updated!", xcp, Logger.ERROR);
                return;
            }
        }
    
        _propertiesToModify.add(property);
                
        if (  ! isAutoPropertyValueCommit() ) return;
     
        commitPropertyValues();
        
    }
    
    // Update local KAoSAgentDescription with the valid updates in the DS, so the KAoSAgentDescription
    // stored in the DS and the local one are in sync.
    private void updateLocalActorDescription(List modifiedProperties)
    {
        try {
            _actorDesc.modifyProperties(modifiedProperties);
        }
        catch(ModificationException e){
            //this should not happen, since the properties have been already validated
            // for now:
            e.printStackTrace();
        }
        
    }
    
    // Return Properties that have been successfully updated in the DS.
    private List getValidPropertyMods(List failedProperties)
    {
        List validMods = new ArrayList();
        for (Iterator propertiesToModifyIt = _propertiesToModify.iterator(); propertiesToModifyIt.hasNext();) {
            Property prop = (Property) propertiesToModifyIt.next();
            boolean foundFailedMatch = false;
            for (Iterator failedIt = failedProperties.iterator(); failedIt.hasNext();) {
                Property failed = (Property)failedIt.next(); 
                if ( failed.getSequenceId().equals(prop.getSequenceId()) ) { 
                    foundFailedMatch = true;
                    break;
                }
            } // end failures
            if ( ! foundFailedMatch)  validMods.add(prop);
        }// end for propertiesToModifyIt
        return validMods;
    }
    
    private List copyProps(List properties)
    {
        List copy = new ArrayList(properties.size());
        for (Iterator propIt = properties.iterator(); propIt.hasNext();) {
            Property prop = (Property)propIt.next(); 
            copy.add(prop);
        } 
        return copy;
    }
    
    //////////////////////////////////////////////////////////////
    // Entry point...

    public static void main(String [] args)
      throws Exception      
    {
    
        KAoSActor agentA = new KAoSActorImpl(args);
       
        synchronized ( Thread.currentThread() ) {
            Thread.currentThread().wait();
        }
    
   }
}

