/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/11/23 17:49:07 $
 */
 
package kaos.core.csi;

import java.util.Set;

/**
 *   This interface containes methods allowing to create and investigate a description of an Ontology property.
 *   
 *   $Revision: 1.2 $
 */

public interface OntPropertyDescription
{    
    /** 
     * Set the value of this property.
     *
     * @param value      String containing the value of the property. 
     */
    public void addValue(String value);
    
    /** 
     * Set the optional description of the given property value.
     * 
     * @param value      String containing the value of the property.
     *
     * @param desc       Object containing the description  of the value
     *                   often it will be kaos.core.csi.OntInstanceDescription 
     */
    public void setValueDescForValue(String value, Object desc);
    
    /** 
     * Get the name of this property.
     *
     * @return           String containing the uri of the property.
     */
    public String getName();
    
    /** 
     * Get the values of this property.
     *
     * @return           Set containing the value of the property. 
     */
    public Set getValues();
    
    /** 
     * Get the optional description of the given property value.
     * 
     * @param value      String containing the value of the property.
     *
     * @return           Object containing the description  of the value
     *                   often it will be kaos.core.csi.OntInstanceDescription 
     */
    public Object getValueDescForValue(String value);
    
    /** 
     * Remove the property value.
     * 
     * @param value      String containing the value of the property.
     */
    public void removeValue(String value);
    
    /** 
     * Remove the optional description of the given property value.
     * 
     * @param value      String containing the value of the property.
     */
    public void removeValueDescForValue(String value);
}