package kaos.core.csi.extension.history.logging;


import kaos.core.csi.ActionInstanceDescription;
import kaos.policy.information.BasicActionDescription;
import kaos.ontology.vocabulary.ActionConcepts;

import kaos.policy.information.KAoSProperty;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.util.Logger;

import java.util.Set;
import java.util.Vector;
import java.util.List;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.HashSet;
import java.util.Enumeration;

import java.rmi.dgc.VMID;

import java.io.Serializable;


public class HistoryMonitorImpl implements HistoryMonitor, Serializable {

	private Hashtable _actorLogs; // logs keyed on actors
	private Hashtable _actionLogs; // logs keyed on actions
	private Vector _allLogs;
	private boolean _dirty = false;
	private String _ontType;
	private String _name;
	private Vector _associatedOntTypes;
	
	private static int _NO_MATCH = 0;
	private static int _COMPLETE_MATCH = 1;
	private static int _PARTIAL_MATCH = 2;
	    						  
	public HistoryMonitorImpl(String ontType)
	{
		_actionLogs = new Hashtable();
		_actorLogs = new Hashtable();
		_allLogs = new Vector();
		_ontType = ontType; //PolicyActionConcepts.PolicyAction();
		_associatedOntTypes = new Vector();
		_associatedOntTypes.add(_ontType);
		
		// globally unique name
		VMID vmid = new VMID();
		_name = vmid.toString();
	}
	
	/******* KAoSExtensionComponent implementation begins **********/
	
	public String getAssociatedOntologyType()
	{
		return _ontType;
	}
	
	public List getOntologicalAttributes()
	{
	    return _associatedOntTypes;
	}
	
	public void setOntologicalConcepts(Vector concepts)
	{
	    _associatedOntTypes = concepts;
	}	
	
	public String getName()
	{
		return _name;
	}
	
	public void setName(String name)
	{
	    _name = name;
	}
	
	public void setEnabledStatus(boolean status)
	{
	    
	}
	
	
	public boolean getEnabledStatus()
	{
	    return false;
	}

	
	/******* KAoSExtensionComponent implementation ends ************/
	
	
	/**
     * Log an event
     * 
     * @param  event	The event to be logged.
     *     
     */
	
	public void logEvent(ActionInstanceDescription event)
	{
		Vector log;
	
		if (_logger.isEnabledFor(Logger.INFO)){
			_logger.logMessage("\n logEvent: received event: " + event, 
							   Logger.INFO);
		}
		String actionType = event.getActionClassName();
		
		if (_logger.isEnabledFor(Logger.INFO)){
			_logger.logMessage("\n logEvent: actionTypet: " + actionType, 
							   Logger.INFO);
		}
			
			log = (Vector)_actionLogs.get(actionType); 
					
			if(log == null)
			{
				log = new Vector();
				_actionLogs.put(actionType, log);
			}
			
			log.addElement(event);			
		
			if (_logger.isEnabledFor(Logger.INFO)){
				_logger.logMessage("\n logEvent: Actions Log: " + _actionLogs, 
								   Logger.INFO);
			}
		
		// this represents the actor property from the Action Instance Description
		// which was submitted to Guard for checking authorization
		
		
		Set actors = event.getProperty(ActionConcepts.performedBy()).getValues();
		
		
        Iterator it;
        
		if(actors != null)
		{		
			it = actors.iterator();
		
			while(it.hasNext())
			{
				String next = (String)it.next();
			
				log = (Vector)_actorLogs.get(next); 
			
				
				if(log == null)
				{
					log = new Vector();
					_actorLogs.put(next, log);
				}
			
				log.addElement(event);			
			}
		}
		else
		{
			// this is not a 'checkPermission' event, so seek the normal actor
			Object actor = event.getActor();

			log = (Vector)_actorLogs.get(actor); 
			
			if(log == null)
			{
				log = new Vector();
				_actorLogs.put(actor, log);
			}					

			log.addElement(event);	
			
		}

		if (_logger.isEnabledFor(Logger.INFO)){
			_logger.logMessage("\n logEvent: Actors Log: " + _actorLogs, 
							   Logger.INFO);
		}
		
		_allLogs.addElement(event);
		
		if (_logger.isEnabledFor(Logger.INFO)){
			_logger.logMessage("\n logEvent: All Log: " + _allLogs, 
							   Logger.INFO);
		}
	}
	
	
	/**
     * Get all the events logged so far.
     *      
     * @return  A Vector containing aid objects logged so far.
     */
	
	public Vector getAllLogs()
	{				
		if (_logger.isEnabledFor(Logger.INFO))
		{
			_logger.logMessage("\n getAllLogs called " + _allLogs.size(), 
							   Logger.INFO);
		}
		
		return _allLogs;
	}
	
	
	/**
     * Delete all the events logged so far.
     *           
     */
	
	public void deleteAllLogs()
	{
		_allLogs = new Vector();
		_actionLogs = new Hashtable();
		_actorLogs = new Hashtable();
	}
	
	public void deleteLogs(Vector logs)
	{
		_allLogs.removeAll(logs);

		// to indicate that we need to update each event set
		// keyed on actors and actions to reflect this latest deletion
		_dirty = true; 		
	}

	/**
     * Get all events logged that contain the given action type.
     * 
     * @param  actionType		A String indicating the action type.
     *
     * @return  A vector containing aid objects logged for that action. 
     */
	
	public Vector getLogsforActionType(String actionType)
	{
		Vector ret = (Vector)_actionLogs.get(actionType);	
		
		if(_dirty)
		{
			// update the set to reflect the latest log deletion
			ret.retainAll(_allLogs);
			_actionLogs.put(actionType, ret);
		}
		
		return 	ret;
	}
	
	
	/**
     * Delete all logged events that contain the given action type.
     * 
     * @param		actionType		A String indicating the action type.
	 * 
	 * @return		Vector			A Vector containing deleted logs
     *     
     */
	
	public Vector deleteLogsforActionType(String actionType)
	{
		Vector logs = (Vector)_actionLogs.remove(actionType);
		
		if(logs == null)
		{
			return null;
		}
		
		_allLogs.removeAll(logs);
		
		// now remove those logs from each actor set as well...
		
		Enumeration keys = _actorLogs.keys();
		
		while(keys.hasMoreElements())
		{
			String actor = (String)keys.nextElement();			
			Vector aLogs = (Vector)_actorLogs.get(actor);
			
			aLogs.removeAll(logs);			
			_actorLogs.put(actor, aLogs);
		}

		return logs;
	}
	
	
	/**
     * Get all logged events that contain the given actor.
     * 
     * @param  actorID		A String indicating the actor id.
     *
     * @return  A vector containing aid objects logged for that actor. 
     */
	
	public Vector getLogsforActor(String actorID)
	{		
		Vector ret = (Vector)_actorLogs.get(actorID);		
		
		if(_dirty)
		{
			// update the set to reflect the latest log deletion
			ret.retainAll(_allLogs);
			_actionLogs.put(actorID, ret);
		}
		
		return ret;
	}
	
	
	/**
     * Delete all logged events that contain the given actor.
     * 
     * @param		actorID	A String indicating the actor id.
	 * 
	 * @return		Vector			A Vector containing deleted logs
     *     
     */
	
	public Vector deleteLogsforActor(String actorID)
	{
		Vector logs = (Vector)_actorLogs.remove(actorID);
		
		if(logs == null)
		{
			return null;
		}
		
		_allLogs.removeAll(logs);
		
		// now remove those logs from each actiontype set as well...
		
		Enumeration keys = _actionLogs.keys();
		
		while(keys.hasMoreElements())
		{
			String action = (String)keys.nextElement();			
			Vector aLogs = (Vector)_actionLogs.get(action);
						
			aLogs.removeAll(logs);			
			_actionLogs.put(action, aLogs);
		}

		return logs;
	}
	              
	
	/** This method returns the logged events specified by the given
	 *  parameters. The input parameters are interpreted as follows:
	 * 	  	 
	 *  @param	actionType	A String indicating the action type of requested logs.
	 *						If it is null, 'all action types' would be
	 *						assumed.
	 *  
	 *  @param  actorID		A String indicating the actor of requested logs.
	 *						If null, 'all actors' would be assumed.
	 * 
	 *	@param	startTime	A long, indicating the start of time period for
	 *						which logs are requested.
	 *						If null, 'the begining of logs' time would be assumed.
	 * 
	 *  @param	endTime		A long, indicating the end of time period for which
	 *						logs are requested.
	 *						If null, 'time of the latest log' would be assumed.
	 * 
	 *	@param	latest		A boolean, indicating if only latest logs are requested.
	 *						If it is set:
	 *						a) start and end time parameters are ignored. 
	 *						b) If actionType is null, and actorID is null, 
	 *						the latest log for each actor is returned. 
	 *						c) If actorID is null and actionType is not null
	 *						the latest logs for each actor for the given
	 *						actionType are returned.
	 *						d) If actorID is not null, the latest log for that
	 *						actor is returned. If actionType also is not null,
	 *						the latest log for that actor and that action is
	 *						returned.
	 * 
	 *						If this boolean is not set, log are retrieved
	 *						based on preceding parameters.	 	 
	 */
	
	public Vector getSpecifiedLogs(String actionType,
								   String actorID,
								   String startTime,
								   String endTime,
								   Boolean latest)
	{		
		if (_logger.isEnabledFor(Logger.INFO))
		{
			_logger.logMessage("\n getSpecifiedLogs called with action: " +
								actionType + " actor: " + actorID + 
								" startTime: " + startTime + 
								" endTime: " + endTime + " latest: " + latest, 
								Logger.INFO);
		}
		
		
			
		if(_allLogs.size() <= 0)
		{
			// no logs present
			
			if(_logger.isEnabledFor(Logger.INFO))
			{
				_logger.logMessage("\n no logs present! returning empty vector",
								   Logger.INFO);
			}
			
			return new Vector();
		}
		
		// by default we return all logs
		Vector finalLogs = _allLogs;
		Vector temp = null;
		
		Vector ret = new Vector();
		
		if(!actorID.equals(""))
		{
			// get logs for the specifed actor
			temp = (Vector)_actorLogs.get(actorID);
			
			if(temp != null)
			{								
				finalLogs = temp;				
			}
		}
		
		if(!actionType.equals(""))
		{
			// get logs only for the specified action types
			temp = (Vector)_actionLogs.get(actionType);
			
			if(temp != null)
			{
				if(actorID != null)
				{
					// this means the 'finalLogs' points to actor logs...
					// take its intersection with actor logs
					temp.retainAll(finalLogs);										
				}
				
				finalLogs = temp;								
			}
		}
						
		// check if the latest logs are required
		
		if(latest.booleanValue())
		{
			if(actorID == null)
			{
				// filter the logs and return the latest log
				// for each actor
				
				// we go from end to begining because the
				// latest logs would be towards the end...
				
				ActionInstanceDescriptionImpl log;				
				
				Set actors = new HashSet();
				
				for(int i = (finalLogs.size() - 1); i >=0; i--)
				{
					log = (ActionInstanceDescriptionImpl)finalLogs.elementAt(i);
										
					Set actorIDs = log.getProperty(ActionConcepts.performedOn()).getValues();
													  
					Iterator it = actorIDs.iterator();
					
					// we assume that the set contains only one element					
					String id = (String)it.next();
					
					if(!actors.contains(id))
					{
						// we don't have the latest log for this actor						
						actors.add(id);
						ret.addElement(log);
					}											
					
				} // end for 
								
				return ret; // return the logs gathered so far
					
			} // end if(actorID == null)
			
			//if we reach here, actor id is non null
			// return the latest log for that actor				
			
			ret.addElement(finalLogs.lastElement());
			
			return ret;
		
		} // end if(latest)
		
		// check if the time window is specified
		
		if(startTime.equals("") && endTime.equals(""))
		{
			// no time window specified
			// return the logs that we filterd so far...
			
			return finalLogs;
		}
		
		long start = 0, end = Long.MAX_VALUE;
		
		if(!startTime.equals(""))
		{
			start = (new Long(startTime)).longValue();
			
			if(_logger.isEnabledFor(Logger.INFO))
			{
				_logger.logMessage("\n start time for log search: " + start,
								   Logger.INFO);
			}			
		}
		
		if(!endTime.equals(""))
		{
			end = (new Long(endTime)).longValue();
			
			if(_logger.isEnabledFor(Logger.INFO))
			{
				_logger.logMessage("\n end time for log search: " + end,
								   Logger.INFO);
			}
		}
		   
		// and now filter the logs based on their time
		
		ActionInstanceDescriptionImpl log;
		
		for(int i = 0; i < finalLogs.size(); i++)
		{
			log = (ActionInstanceDescriptionImpl)finalLogs.elementAt(i);
			
			Set values = log.getProperty(ActionConcepts.hasBeginTime()).getValues();
			
			Iterator it = values.iterator();
			
			String t = (String)it.next(); // a single time value
			
			long time = (new Long(t)).longValue();
			
			if((time >= start) && (time <= end))
			{				
				if(_logger.isEnabledFor(Logger.INFO))
				{
					_logger.logMessage("\n this log time: " + time + " in time window",
									   Logger.INFO);
				}
				ret.addElement(log);
			}
		}
		
		return ret;
	}
	
	
	/** This method calls 'getSpecifiedLogs' with its input parameters
	 *  and then deletes the logs returned by that call.
	 * 
	 *	For method documentation, see the heading for 'getSpecifiedLogs'.
	 * 
	 *  @return		Vector			A Vector containing deleted logs
	 */
	
	public Vector deleteSpecifiedLogs(String actionType,
								   String actorID,
								   String startTime,
								   String endTime,
								   Boolean latest)
	{
	
		Vector del = getSpecifiedLogs(actionType, actorID, startTime, 
									  endTime, latest);
		
		_allLogs.removeAll(del);		
		
		// to indicate that we need to update each event set
		// keyed on actors and actions to reflect this latest deletion
		_dirty = true; 

		return del;
	}
	
	/** This method is used to find out the # occurrences of 
	 *  actual events of the type indicated by an event class.
	 * 
	 *  @param		eventClass	A BasicActionDescription object specifying
	 *							the type of event class.
	 * 
	 *	@return		the # events in this log that are instances of the
	 *				specified event class.
	 */
	
	public int countEventsOfType(BasicActionDescription eventClass)
	{
		int counter = 0;
		
		for(int i = 0; i < _allLogs.size(); i++)
		{
			ActionInstanceDescriptionImpl log = (ActionInstanceDescriptionImpl)_allLogs.elementAt(i);
			
			if(classifyEvent(eventClass, log) == _COMPLETE_MATCH)
			{
				counter++;
			}
		}

		return counter;
	}
	
	/** This method checks if the given event is an instance of the
	 *  specified event class
	 * 
	 *  @param		eventClass	a BasicActionDescription object specifying
	 *							the event class
	 * 
	 *	@param		event		an ActionInstanceDescription object specifying
	 *							the event
	 * 
	 *	@return		an int interpreted as follows:
	 *				0 : NO_MATCH
	 *				1 :	COMPLETE_MATCH
	 *				2 : PARTIAL_MATCH
	 */
	
	private int classifyEvent(BasicActionDescription eventClass,
							  ActionInstanceDescription event)
	{
		boolean partial = false;
        int result;
        
		if (_logger.isEnabledFor(Logger.INFO))
            _logger.logMessage("classifyEvent: Start\n\n    Event: \n" + event.toString() 
		                + "\n\n   Event class information: \n\n" + eventClass, Logger.INFO);
				
       /* pre-new AID:
		
		OntPropertyDescription instaction = event.getProperty(ActionConcepts.Action());
		 Set originalAction = instaction.getValues(); 
 		
	        KAoSProperty actionCheck = new KAoSProperty(ActionConcepts.Action());
	        actionCheck.addMultipleInstances(originalAction);
	     */   
	      
		Set originalAction = new HashSet();
        originalAction.add(event.getActionClassName());
        
        		
        KAoSProperty actionCheck = new KAoSProperty(ActionConcepts.Action());
        actionCheck.addMultipleInstances(originalAction);
        
        if (_logger.isEnabledFor(Logger.INFO))
            _logger.logMessage("classifyEvent:\n ***** Event_Action_ property: " 
                           + originalAction, Logger.INFO);
				
		// check if the action specified by event class matches with 
		// the action of this event
		
     
        result = eventClass.compareProperty(actionCheck);
		
		/*  In the following comparison of Action properties, a partial match is considered a complete match. 
		    A partial match is when only a subset of instances of the ActionConcepts._Action_ property described
		    by ActionInstanceDescription is contained by the instances of the ActionConcepts._Action_ property
		    described by PolicyInformation.
		    
		    The reason why the partial match is treated as complete match is because in PolicyInformation
		    the instances of ActionConcepts._Action_ property contain only the policy action, ex:
		    
		    http://ontology.coginst.uwf.edu/Action.daml#CommunicationAction
		    
		    , whereas the instances of ActionConcepts._Action_ in ActionInstanceDescription *may* also contain
		    all superclasses of the given action, ex:
		    
		    [|http://ontology.coginst.uwf.edu/Action.daml#|::|Action|, |http://ontology.coginst.uwf.edu/Action.daml#|::|CommunicationAction|,
		    |http://ontology.coginst.uwf.edu/Action.daml#|::|Target|, |http://www.daml.org/2001/03/daml+oil#|::|Thing|, 
		    |Anon_1051106261912_26|]
		    
		    This is  why the ActionConcepts._Action_  property is checked outside the following 'while' loop, 
		    because its partial result would lead to the processing of incomplete match and it has to be
		    processed as a complete match.
		    
		*/
		
        if( result == KAoSProperty._PROPERTY_NOT_FOUND ||
            result == KAoSProperty._NO_INST_PRESENT ) {           
            //           if (gotSuperActions) {        
            // this policy does not control this action or its superclasses...
            // hence return saying this does not match...
            if (_logger.isEnabledFor(Logger.INFO))
                _logger.logMessage("classifyEvent: \n ***** This action is not a match!", Logger.INFO);
            return _NO_MATCH;
        }
		
        if (_logger.isEnabledFor(Logger.INFO))
            _logger.logMessage("classifyEvent: \n ***** This action matched!", Logger.INFO);
		
        Enumeration classProps = eventClass.getAllProperties();		
		
		while(classProps.hasMoreElements()) 
		{
            KAoSProperty instprop = (KAoSProperty)classProps.nextElement();
			
            if(ActionConcepts.Action().equals(instprop.getPropertyName()))
			{
                // we already checked the action, so continue...
                continue;
            }
			
            if (_logger.isEnabledFor(Logger.INFO))
                _logger.logMessage("classifyEvent:\n ***** Comparing event class property " + instprop.toString(), Logger.INFO);
			
            // we now check if the instances from action description property
            // are in the policy property, hence comparePropertyOtherWay!
            
			/*
			 *  12-20-05: Commented out next statement to follow the same logic as in the Guard
			 */
           // result = instprop.comparePropertyOtherWay(event);
                 
            result = BasicActionDescription.testPropertyPresenceInAID(event,instprop); 	
            if( result == KAoSProperty._NO_INST_PRESENT ) 
			{
                // No targets of actionDesc are present in the policy targets 
                // for this property.
                // return with a 'no match'
                if (_logger.isEnabledFor(Logger.INFO))
                    _logger.logMessage("classifyEvent:\n ***** Targets of event are *not* present in event class targerts for this property!", Logger.INFO);
				
                return _NO_MATCH;
            }						
			
            if( result == KAoSProperty._ALL_INST_PRESENT ) 
			{
                // we had a complete match...
                // continue with the next property
                if (_logger.isEnabledFor(Logger.INFO))
                    _logger.logMessage("classifyEvent:\n ***** This property is completely present!", Logger.INFO);
				
                continue;
            }
						
            // A partial match for this property...
            // This means that the actionDesc either does not have a policy property
            // or, for some property, only some of the actionDesc's targets match 
            // policy's targets.
            // Set the flag and continue
            partial = true;
			
			String msgInfo = "***** This event class property is absent in event or, \n";
			msgInfo = msgInfo + " only some targets in event match event class targets.";
			
            if (_logger.isEnabledFor(Logger.INFO))
                _logger.logMessage("classifyEvent:\n " + msgInfo, Logger.INFO);
        }
				
        if ( partial ) 
		{
            return _PARTIAL_MATCH;
        } 
        else 
		{
            return _COMPLETE_MATCH;
        }
		
	}
	
	/**
     * A utility method to print a String.  Add the name of the class printing the String.
     * 
     * @param msg           The String to be printed.
     */ 
    private void printDebugString (String msg, int loggerDisplayLevel) 
    {    
        _logger.logMessage("**** " + this.getClass().getName() + "\n                " + msg, loggerDisplayLevel);
    }
    
    private static Logger _logger = Logger.getLogger(HistoryMonitorImpl.class);
}
