/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 15:32:57 $
 */

package kaos.core.csi.policy;

import java.util.List;
import java.util.Vector;

/**
 *   KAoSSecurityException defines an exception to be thrown by the Guard if there
 *   is no permission ot perform a given action
 *
 *   @author         KAoS Team
 *   $Revision: 1.3 $
 */
public class KAoSSecurityException extends SecurityException 
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3821454219399307582L;
    
    // constants
    public static final String OBLIGATIONS_NOT_ATTEMPTED = "Original action not permitted.  Obligations not attempted.";
    public static final String OBLIGATIONS_FAILED_INCONSISTENT_STATUS = "Original action permitted. Obligation status inconsistent with action.";
    public static final String OBLIGATIONS_FAILED_SERVICE_FAILURE = "Original action permitted. Service required by obligations failed.";
    public static final String OBLIGATIONS_FAILED_ACTOR_NOT_LOCATABLE = "Original action permitted. Obligation actor not locatable.";
        
    /**
     *  Policy id, which caused the exception
     */
    public String forbiddingPolicyId;
    
    /**
     *  Flag indicating if the default authorization was used
     */
    public boolean defaultAuthorization;
    
    /**
     *  Flag indicating if the authorization check was interrupted and thus the negative authorizaton was returned
     */
    public boolean authorizationCheckInterupted;
	
	private Vector _forbiddingPolicies = null;
	
	// Vector containing obligations that have not been fulfilled.
	private List _unfulfilledObligations = null;
	
	private String _reasonForUnfulfilledObligations;
	
	
    
    /**
     *   Constructs a new KAoSSecurityException with no detail message.
     */
    public KAoSSecurityException () 
    {     
        super(); 
    }

    /**
     *   Constructs a new KAoSSecurityException with a detail message.
     *   A detail message is a String that describes the exception.
     */
    public KAoSSecurityException (String s) 
    {    
        super(s); 
    }
	
	/** add a list of all A- policies forbidding an action
	 */
	
	public void setForbiddingPolicyIDs(Vector v)
	{
		_forbiddingPolicies = v;
	}
	
	/** get the first forbidding policy id from the vector
	 */
	
	public String getFirstForbiddingPolicy()
	{
		if(_forbiddingPolicies == null)
		{
			return null;
		}
		
		return (String)_forbiddingPolicies.elementAt(0);
	}
	
	public Vector getAllForbiddingPolicies()
	{
		return _forbiddingPolicies;
	}
	
	public List getUnfulfilledObligations()
	{
	    return _unfulfilledObligations;
	}
	
	public void setUnfulfilledObligations(List obligations)
	{
	    _unfulfilledObligations = obligations;
	}
	
	public String getReasonForUnfulfilledObligations()
	{
	    return _reasonForUnfulfilledObligations;
	}
	
	public void setReasonForUnfulfilledObligations(String reason)
	{
	    _reasonForUnfulfilledObligations = reason;
	}
	
	
}
