/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/16 18:41:44 $
 */

package kaos.core.csi.policy;

import java.util.ArrayList;
import java.util.List;
import kaos.core.csi.ActionInstanceDescription;
import kaos.core.util.NamespaceValidator;

/**
 *   ObligationViolationException defines an exception, which is thrown when the evaluation of
 *   a trigger against applicable obligation policy determines that some part of obligation 
 *   constraint has been violated, or that for some reason an obligation cannot be fulfilled.
 *
 *   @author         KAoS Team
 *   $Revision: 1.2 $
 */
public class ObligationViolationException extends Exception 
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = -4880938040515915768L;
    private List _violatedObligations;

    /**
     *   Constructs a new ObligationViolationException with no detailed message.
     */
    public ObligationViolationException () { 
        
        super(); 
        
    }

    /**
     *   Constructs a new ObligationViolationException with a detailed message.
     *   A detail message is a String that describes the exception.
     *
     *   @param msg      String containning the detailed message.
     */
    public ObligationViolationException (String msg) { 
        
        super(msg); 
    }
    
    /** 
     * Add an ActionInstanceDescription describing an violated obligation. 
     *
     * @param violatedObligation    	ActionInstanceDescription of the violated obligation
     * 									to be added to the list.
     * 
     */
    public void addUnfulfilledObligation(ActionInstanceDescription violatedObligation )
    { 
        if ( _violatedObligations == null )
        	_violatedObligations = new ArrayList();

        _violatedObligations.add(violatedObligation);
    }

    public List getViolatedObligations()
	{
	    return _violatedObligations;
	}
    
    /** 
     * Set _violatedObligations to the given List.  This operation will replace the existing _violatedObligations.
     *
     * @param violatedObligations   	List of	ActionInstanceDescriptions describing violated obligations.
     * 
     */
    public void setViolatedObligations(List violatedObligations )
    { 
        if ( violatedObligations == null ) return;
       _violatedObligations = violatedObligations;

    }

	
    
}
