package kaos.core.csi.policy;

import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;


public class ViolatedObligationDescriptionImpl 
	extends ActionInstanceDescriptionImpl 
	implements ViolatedObligationDescription {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1550096279899205758L;

	public static ViolatedObligationDescription createViolatedObligationDesc(ActionInstanceDescription fromAID)
	{
		Set types = fromAID.getTypes();
		String actionClassName = null;
		if ( types != null ){
		    for (Iterator typesIt = types.iterator(); typesIt.hasNext(); ){
		    	actionClassName = (String)typesIt.next();
		    	break;
		    }
		}
		
		ViolatedObligationDescription desc = null;
			
		try{
			desc =	new ViolatedObligationDescriptionImpl(fromAID.getName(),
														  actionClassName,
														  fromAID.getActor());
		}
		catch(Exception e){
			e.printStackTrace();
			return null;
		}
		
		Set fromAIDprops = fromAID.getProperties();
		if ( fromAIDprops != null && fromAIDprops.size() > 0 ){
			for (Iterator propsIt = fromAIDprops.iterator(); propsIt.hasNext(); ){
		    	OntPropertyDescription propDesc = (OntPropertyDescription)propsIt.next();
		    	desc.addProperty(propDesc);
		    }
		}
		return desc;
		
	}
	 /**
     * Create an instance of the ViolatedObligationDescription described by the given arguments.
     *
     * @param  actionId             String containing the id of the given action.
     *
     * @param  actionClassName    	String containing the ontological name of the given action.
     * 
     * @param  actor              	String containing the id of the actor performing the action.
     *                               
     * @throws                      URISyntaxException if the argument to create an URI is syntactically incorrect .
     * 
     */
    public ViolatedObligationDescriptionImpl (String actionId, String actionClassName, String actor) throws URISyntaxException
    {
        super(actionId, actionClassName, actor);       
        
    }
	
    /**
     * Create an instance of the ViolatedObligationDescription described by the given arguments.
     *
     * @param actionId             	String containing the id of the given action.
     *
     * @param actionClassName       String containing the ontological name of the given action.
     * 
     * @param actor              	String containing the id of the actor performing the action.
     * 
     * @param properties            HashMap containing the ontological names and their values of properties 
     *                              of the given action.
     *                               
     * @throws                      URISyntaxException if the argument to create an URI is syntactically incorrect .
     * 
     */
    public ViolatedObligationDescriptionImpl (String actionId, 
    									      String actionClassName, 
    									      String actor, 
    									      HashMap properties)
    throws URISyntaxException
    {
        super(actionId, actionClassName, actor, properties);
          
    }
    
    /**
     * Create an instance of the ViolatedObligationDescription described by the given OWL String.
     *
     * @param  owlString            String the action id, class name, actor id and, optionally, properties.
     *                               
     * @throws                      URISyntaxException if the argument to create an URI is syntactically incorrect .
     * 
     */
    public ViolatedObligationDescriptionImpl (String owlString)
    throws URISyntaxException
    {
    	//TODO Parse the OWL String to retrieve the action id, class name, properties (if present) and actor id.
    	super(null);
    }
    
	public String getReasonForViolation() {
		OntPropertyDescription reasonDesc = getProperty(OntPropertyDescriptionImpl.OBLIGATION_VIOLATION_PROPERTY_NAME);
		if ( reasonDesc != null ) {
			Set values =  reasonDesc.getValues();
			if ( values != null ){
			    for (Iterator valuesIt = values.iterator(); valuesIt.hasNext(); )
				  return (String)valuesIt.next();
			}
		}
		return null; 
			
	}
	
	/**
     *   Set the reason for violating  this obligation.
     */
    public void setReasonForViolation(String reason){
    	OntPropertyDescription violatedObligationProp = 
    		OntPropertyDescriptionImpl.createObligationViolationProperty(reason);
     	    addProperty(violatedObligationProp);
    }

}
