/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2004/11/17 23:09:16 $
 */
 
package kaos.core.csi.query;

import kaos.core.service.directory.DomainDescription;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.ontology.util.QueryFailure;

import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.directory.SearchException;

import java.util.Hashtable;
import java.util.List;

/**
 * Defines the operations to query the different aspects of domain and actor registrations
 * 
 * @author KAoS Team
 * $Revision: 1.5 $
 */
public interface QueryRegistration
{
    /**
     * Returns description of the domain identified by the given name
     *
     * @param name             String specifying the name of the domain to lookup
     *
     * @return                 DomainDescription identified by the given name
     *
     * @throws                 DirectoryFailure if the connection to the DS cannot be established.
     *
     */
    public DomainDescription getDomain(String name) throws DirectoryFailure;
    
    /**
     * Returns description of the actor identified by the given name
     *
     * @param actorGUID        String specifying the GUID of the actor to lookup
     *
     * @return                 KAoSAgentDescription identified by the given name
     *
     * @throws                 DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                 SearchException if the actor with the given name 
     *                         has not been found in the DS
     */
    public KAoSAgentDescription  getActor(String actorGUID) throws DirectoryFailure, SearchException;
    
    /**
     * Returns description of the actors fullfiling constraints expressed as the <a href="http://www-ksl.stanford.edu/knowledge-sharing/kif/">KIF</a> statemant
     *
     * @param actorConstraints String specifying the GUID of the actor to lookup
     * An example of a Constraints statement: 
     *                         Give me all agents that have as capability 'take picture' with a cost of 
     *                         less than $1,- with a reliability of 75% or better and where the average number 
     *                         of processed messages in the past hour is less than 20.
     *
     * @return                 KAoSAgentDescription identified by the given name
     *
     * @throws                 DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                 NoSuchActorException  if no actors matching the query properties could be found
     * 
     * @throws                 QueryFailure if query failed
     * 
     */
    public List  getActorsFulfilling(String actorConstraints) throws DirectoryFailure, NoSuchActorException, QueryFailure;
    
    
    /**
     * Returns descriptions of the actors whose KAoSAgentDescription contains all specified attributes 
     * and their values.
     *
     * @param attributes        Hashtable containing attribute names and their values to match the attributes in the DS.
     *
     * @return                  List containing  KAoSAgentDescriptions matching the attributes.
     *
     * @throws                  DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                 QueryFailure if query failed
     * 
     */
    public List  getActorsWithAttributes(Hashtable attributes) 
        throws DirectoryFailure, QueryFailure;
                
    /**
     * Return all agent ids stored in the KAoS directory.
     *
     * @return List         of all agent unique IDs in the agent directory, or null, if there aren't any.
     *
     * @throws              SearchException if an exception is thrown during the search.
     *
     * @throws              DirectoryFailure if the directory cannot be found/is not available.
     */    
    public List getAllAgents()
        throws SearchException, DirectoryFailure;

 }
