/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/11/23 17:49:07 $
 */
 
package kaos.core.csi.registration;

import kaos.core.service.directory.DomainDescription;
import kaos.core.service.directory.HasParentException;

import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.directory.NotRegisteredException;


/**
 * Defines the operations to configure the parent-child domain relationship for the specified domain.
 * 
 * @author KAoS Team
 * $Revision: 1.3 $
 */
public interface DomainHierarchyConfiguration
{
    /** 
     * Set as a the parent the given parentDomain for the specified domain. 
     *
     * @param domain             DomainDescription describes the domain, whose parent is to be updated.
     *
     * @param parentDomain       DomainDescription describes the parent domain.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   HasParentException if the specified domain has already a parent. 
     *                           
     *
     */
    public void setParentForDomain(DomainDescription domain,
                                   DomainDescription parentDomain )
    throws DirectoryFailure,
          HasParentException;

    /** 
     * Add the childDomain as a child of the specified domain.
     *
     * @param domain             DomainDescription describes the domain, to which the childDomain will be added.
     *
     * @param childDomain        DomainDescription describes the child domain.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   HasParentException if the specified child domain has already a parent. 
     *                           
     *
     */
    public void addChildToDomain(DomainDescription domain,
                                 DomainDescription childDomain )
    throws DirectoryFailure,
           HasParentException;
           
           
    /** 
     * Add the childDomain as a child of the specified domain.
     *
     * @param domain             DomainDescription describes the domain, from which to remove the  child domain.
     *
     * @param childDomain        DomainDescription describes the child domain to be removed.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   NotRegisteredException if the specified domain does not have the specified child domain as a child.
     *                           
     *
     */
    public void removeChildFromDomain(DomainDescription domain,
                                      DomainDescription childDomain )
    throws DirectoryFailure,
           NotRegisteredException;

 }
