/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/11/23 17:49:07 $
 */
 
package kaos.core.csi.registration;

import kaos.core.service.directory.DomainDescription;
import kaos.core.service.directory.KAoSEntityDescription;

import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.directory.NotRegisteredException;


/**
 * Defines the operations to register and deregister entities in the specified domain.
 * 
 * @author KAoS Team
 * $Revision: 1.2 $
 */
public interface DomainRegistration
{

   /** 
    * Register the entity described by the given KAoSEntityDescription in the specified domain. 
    *
    * @param domain             DomainDescription describing the domain, in which to register the given entity.
    *
    * @param entityDesc         KAoSEntityDescription describes the entity to be registered in the given domain.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering entity has been already registered in the 
    *                           the given domain.
    *
    */
   public void registerEntityInDomain(DomainDescription domain,
                                      KAoSEntityDescription entityDesc)
   throws DirectoryFailure,
          AlreadyRegisteredException;

  /** 
    * Deregister the entity described by the given KAoSEntityDescription from the specified domain. 
    *
    * @param domain             DomainDescription describing the domain, from which to deregister the given entity.
    *
    * @param entityDesc         KAoSEntityDescription describes the entity to be deregistered from the given domain.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   NotRegisteredException if the deregistering entity has not been registered in the 
    *                           the given domain.
    *
    */
   public void deregisterEntityFromDomain(DomainDescription domain,
                                          KAoSEntityDescription entityDesc)
   throws DirectoryFailure,
          NotRegisteredException;

 }
