package kaos.core.csi.registration;

import java.io.StringWriter;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import jtp.ReasoningException;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.rdf.model.Resource;

import kaos.ontology.repository.OntologyRepository;
import kaos.ontology.util.SerializableOntModelImpl;
import kaos.ontology.vocabulary.OWLConcepts;
import kaos.ontology.vocabulary.RDFConcepts;

/**
 * 
 * @author lbunch
 *
 * Utility class for manipulating kaos.core.csi.Property objects
 */
public class PropertyUtil
{
	/**
	 * Create list of OWL individuals identified by entityGUID with the given property values assigned
	 * 
	 * @param entityGUID	globally unique identifier String for an ontology individual
	 * @param properties	List of properties (name-value pairs)
	 * @return List         of OWL strings containg the individual with the given id and properties
	 */
	public static List getPropertiesAsOWL(String entityGUID, List properties, OntologyRepository ontRepository) throws ReasoningException
	{
		if (entityGUID == null || entityGUID.length() == 0 || properties == null || properties.size() == 0)
			return null;
		
		List myResult = new ArrayList();
		
		for(Iterator propIter = properties.iterator(); propIter.hasNext();)
		{
			Property prop = (Property)propIter.next();
			myResult.add(getPropertyAsOWL(entityGUID, prop, ontRepository));
		}
		
		return myResult;
	}
	
	/**
	 * Create an OWL individual identified by entityGUID with the given property values assigned
	 * 
	 * @param entityGUID	globally unique identifier String for an ontology individual
	 * @param myProperty	Property (name-value pair)
	 * @return String       containg the OWL definition of individual with the given id and property
	 */
	public static String getPropertyAsOWL(String entityGUID, Property myProperty, OntologyRepository ontRepository) throws ReasoningException
	{
		if (entityGUID == null || entityGUID.length() == 0 || myProperty == null || myProperty.getName() == null || myProperty.getValue() == null)
			return null;
		
		return getTripleAsOWL(entityGUID, myProperty.getName(), myProperty.getValue().toString(), ontRepository);
	}

	/**
	 * Create an OWL individual identified by teh given triple
	 * 
	 * @param entityGUID	globally unique identifier String for an ontology individual
	 * @param myProperty	property name as String
	 * @param myValue    	property value name as String
	 * @return String       containg the OWL definition of individual with the given id, property and its value
	 */
	public static String getTripleAsOWL(String entityGUID, String myProperty, String myValue, OntologyRepository ontRepository) throws ReasoningException
	{
		if (entityGUID == null || entityGUID.length() == 0 || myProperty == null || myValue == null)
			return null;
		
		SerializableOntModelImpl om = new SerializableOntModelImpl();
		Resource thing = om.createClass(OWLConcepts.Thing());
		Individual indiv = om.createIndividual(entityGUID, thing);
		com.hp.hpl.jena.rdf.model.Property p = om.createProperty(myProperty);
		
		if (ontRepository.testTrue("(" + RDFConcepts._type_ + " " + myProperty + " " + OWLConcepts.DatatypeProperty() + ")"))
			indiv.addProperty(p, myValue);
		else
		{
			Resource r = om.createResource(myValue);
			indiv.addProperty(p, r);
		}
			
		StringWriter sw = new StringWriter();
		om.write(sw, "RDF/XML-ABBREV");
		
		return sw.toString();
	}
}