/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/10 18:48:10 $
 */

package kaos.core.csi.request;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.request.ActionFailureException;
import kaos.core.service.ontology.request.ActionStatusCallback;


/**
 * Defines the operations to process  a request expressed by an ActionInstanceDescription. 
 * 
 * @author KAoS Team
 */
public interface AIDRequestReceiver 
{
    /**
	 * This method accepts a request described by an ActionInstanceDescription to perform an action.
	 * Feedback from the action execution is sent to the requestor via the given ActionStatusCallback.
	 * 
	 * @param	    aid                     is an ActionInstanceDescription that describes the action.  
	 *							            It includes the action name and all action properties and their values.
	 * 
	 * @param		statusCallback		    ActionStatusCallback to be called to pass back to the 
     *                                      action requestor information about progression of the action execution
     *                                      and the final result.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
    public void processAIDRequest(ActionInstanceDescription aid,
                                  ActionStatusCallback statusCallback)
		throws ActionFailureException;
		
	/**
	 * This method accepts a request described in an ActionInstanceDescription to perform an action.
	 * The result is returned synchronously.
	 * 
	 * @param	    aid                     is an ActionInstanceDescription that describes the action.  
	 *							            It includes the action name and all action properties and their values.
	 * 
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
    public Object processAIDRequest(ActionInstanceDescription aid)
        throws ActionFailureException;

}
