/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/12/12 18:21:15 $
 */
package kaos.core.csi.request;

import java.lang.reflect.Method;
import java.util.Enumeration;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.KAoSActorImpl;
import kaos.core.util.Logger;
import kaos.core.util.MethodCallRequestMsg;
import kaos.core.util.MethodCallResultMsg;
import kaos.core.util.Msg;
import kaos.core.util.MsgUtils;
import kaos.kpat.applet.JenaListCellRenderer;

import javax.agent.TransportMessage;
import javax.agent.service.transport.MessageListener;
import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 *
 * Defines the operations to process an obligation request described by the given obligation
 * ActionInstanceDescription.  In addition, the obligation trigger is given as well to provide
 * more information about the context, in which the trigger action has occurred.
 * 
 * @author KAoS Team
 */

public abstract class ObligationRequestReceiverImpl
	extends KAoSActorImpl
	implements ObligationRequestReceiver, MessageListener
{
	
    /**
     * Instantiates a new KAoS-aware agent using JAS services for transport, naming and 
     * agent directory.
     */
    public ObligationRequestReceiverImpl(String [] args)
    throws Exception
    {
    	super(args);
    }
    
	///////////////////////////////////////////////////////////////////////////////////////////
    // Implementation of the kaos.core.csi.request.ObligationRequestReceiver interface - begin
    
	/**
	 * This method accepts an obligation request to execute by the ObligationRequestReceiver.  The obligation is described
	 * by the given ActionInstanceDescription.  Also the trigger  ActionInstanceDescription is given for the contextual
	 * information of the trigger action.
	 * 
	 * @param       obligationAID		 	is an ActionInstanceDescription containing the obligation action name
	 * 										and its properties.
	 * 
	 * @param		triggerAID	            is an ActionInstanceDescription containing the trigger action name
	 * 										and its properties. 
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
	public void processObligationRequest(ActionInstanceDescription obligationAID, ActionInstanceDescription triggerAID)
		throws ActionFailureException
	{
		if ( obligationAID == null || triggerAID == null )
	        throw new ActionFailureException("Required argument is null\n\tobligationAID: " + obligationAID
	        		                         + "\n\ttriggerAID: " + triggerAID);
	     
	    String actionName = obligationAID.getActionClassName();
		if (_logger.isEnabledFor(Logger.INFO))
		    _logger.logMessage("getActionMethodNameFromAID: got long name: " + actionName, Logger.INFO);
		String methodName = JenaListCellRenderer.getShortName(actionName);
		if (_logger.isEnabledFor(Logger.INFO))
		    _logger.logMessage("getActionMethodNameFromAID: got short name: " + methodName, Logger.INFO);
		
	    if ( methodName == null ) 
	        throw new ActionFailureException("Action name is null, unable to continue processing of:\n " + obligationAID );
	        
	    Object result = null;
	    try {
	    	result = executeObligationAction(methodName, 
	            							 obligationAID, 
	            							 triggerAID);
	    }
	    catch(Exception e) {
	    	if (e instanceof ActionFailureException) {
	    		throw (ActionFailureException) e;
	    	}
		    else {
		    	e.printStackTrace();
		    	throw new ActionFailureException(e.getMessage());
		    }
	    }	    
	}
	//////////////////////////////////////////////////////////////////////////////////////////
    // Implementation of the kaos.core.csi.request.ObligationRequestReceiver interface - end
	
	///////////////////////////////////////////////////////////////////////////////
    // Implementation of the javax.agent.service.transport.MessageListener - begin

    /**
     * Implements the receiveMessage method mandated by the
     * MessageListener interface.  
     * 
     * Sends a reply, if requested, back to the receiver.
     * 
     */
    public void receiveMessage(TransportMessage tmsg)
    {
    	if (_logger.isEnabledFor(Logger.INFO))
            _logger.logMessage("receiveMessage:  ******* received transport message:\n" + tmsg, Logger.INFO);
    
        Msg msgContent = MsgUtils.unwrapMsgContent(tmsg);

        if (msgContent instanceof MethodCallRequestMsg) {
            MethodCallRequestMsg content = (MethodCallRequestMsg) msgContent;
            if (_logger.isEnabledFor(Logger.INFO))
                _logger.logMessage("receiveMessage:  ******* retrieved message content:  \n" + content, Logger.INFO);

            MethodCallResultMsg reply = satisfyRequest(content);

            if ( ! content.isReplyRequested() ) {
            	return;
            }
            
            if ( reply == null ) {
                if (_logger.isEnabledFor(Logger.INFO))
                    _logger.logMessage("receiveMessage:  reply is null and will NOT be returned to the caller! \n", Logger.INFO);
                return;
            }
            
            if (_logger.isEnabledFor(Logger.INFO))
                _logger.logMessage("receiveMessage:  ******* sending reply message: \n" + reply, Logger.INFO);
   
            try {
                 _transportSupport.sendMessageTo(_actorDesc,
                		 						 tmsg,
                		 						 reply);
            }
            catch(TransportFailure e){
                if (_logger.isEnabledFor(Logger.WARN))
                    _logger.logMessage("", e, Logger.WARN);
            }
            catch(NotLocatableException e2){
                if (_logger.isEnabledFor(Logger.WARN))
                    _logger.logMessage("", e2, Logger.WARN);
            }
        }
        else {
            handleMsgContent(msgContent);
        }
    	
    }

    
    //////////////////////////////////////////////////////////////////////////////
    // Implementation of the javax.agent.service.transport.MessageListener - end
   
    private Object executeObligationAction(String methodName,
    									   ActionInstanceDescription obligationAID,
    									   ActionInstanceDescription triggerAID) 
    throws Exception
    {
    	if ( methodName == null ||  obligationAID == null || triggerAID == null)  
    		throw new Exception("Required argument is null! Unable to proceed!\n\tmethodName: " 
    							+ methodName + "\n\tobligationAID: " + obligationAID + "\n\ttriggerAID: " + triggerAID);  
    	
    	Object result = null;

    	if (_logger.isEnabledFor(Logger.INFO))
    		_logger.logMessage("executeObligationAction:  action name: " + methodName, Logger.INFO);
 	
    	try {
    		List argObjectList = new ArrayList();
    			
    		argObjectList.add(obligationAID);
    		argObjectList.add(triggerAID);
		
    		Object argObjects[]= argObjectList.toArray( new Object[] {} );
    		result = executeMethodWithArgs(methodName, argObjects);
    		/*
    		// we have to check this the "hard way" because Class.getMethod(name, args[])
    		// does not work if the args are subclasses of the parameter classes
	
    		// find the method matching the method name requested
    		Method methods[] = this.getClass().getMethods();
    		boolean foundMethod = false;
    		for (int i=0; i<methods.length; i++) {
    			if ( methods[i].getName().equals(methodName) ) {
    				// check to see if the parameter classes match the arguments
    				Class parameters[] = methods[i].getParameterTypes();
    				boolean classesMatch = false;
    				if ( parameters.length == argObjects.length) {
    					classesMatch = true ;
    					for (int j=0; j<parameters.length; j++) {
    						if (!parameters[j].isInstance(argObjects[j])) {
    							classesMatch = false;
    							break;
    						}
    					}
    				}
    				// if we found the right method, execute it
    				if ( classesMatch ) {
    					foundMethod = true;
    					result = methods[i].invoke(this, argObjects);
    					break;
    				}
    			}
    		}
    		// if we didn't find the method, throw an exception
    		if ( ! foundMethod ) {                    
    			throw new NoSuchMethodException(methodName);
    		}
    		*/
    	}           		
    	catch (Exception xcp) {
    		if (_logger.isEnabledFor(Logger.WARN))
    			_logger.logMessage("", xcp, Logger.WARN);
    		throw xcp;	
    	}
    	return result;
    }

   private Object executeMethodWithArgs(String methodName, Object [] argObjects)
   throws Exception
   {
	   // we have to check this the "hard way" because Class.getMethod(name, args[])
	   // does not work if the args are subclasses of the parameter classes
	   Object result = null;
	   Method methods[] = this.getClass().getMethods();
	   boolean foundMethod = false;
	  // try {
		   for (int i=0; i<methods.length; i++) {
			   if ( methods[i].getName().equals(methodName) ) {
					// check to see if the parameter classes match the arguments
				   Class parameters[] = methods[i].getParameterTypes();
				   boolean classesMatch = false;
				   if ( parameters.length == argObjects.length) {
					   classesMatch = true ;
					   for (int j=0; j<parameters.length; j++) {
						   if (!parameters[j].isInstance(argObjects[j])) {
							   classesMatch = false;
							   break;
						   }
					   }
				   }
				   // if we found the right method, execute it
				   if ( classesMatch ) {
					   foundMethod = true;
					   result = methods[i].invoke(this, argObjects);
					   break;
				   }
			   }//end if ( methods[i].getName().equals(methodName)
		   }// end for
	  // }
	  // catch(Exception e){
	//	   throw e;
	  // }
	   // if we didn't find the method, throw an exception
	   if ( ! foundMethod ) {                    
		   throw new NoSuchMethodException(methodName);
	   }      		
	   return result;
   }
   
   private void handleMsgContent(Msg content)
   {
	   _logger.logMessage("handleMsgContent: unable to handle msgContent:"
               + content, Logger.WARN);
   }
  

   private MethodCallResultMsg satisfyRequest (MethodCallRequestMsg requestMsg)
   {
	   /*
	    * Currently, there is only 1 method in the ObligationRequestReceiver, which is being asked
	    * to satisfy, so the processing could be just the retrieve the method name, make sure that it is
	    * "processObligationRequest" then retrieve the args, make sure that they are
	    * 2 instances of ActionInstanceDescription and call:
	    * "processObligationRequest(ActionInstanceDescription obligationAID, ActionInstanceDescription triggerAID)".
	    *  However, if the ObligationRequestReceiver evolves then it would not work and that is why
	    *  reflection is used even though for now it is an overwhead. 
	    */
       MethodCallResultMsg resultMsg = null;
       Object result = null;
       try {         
           String methodName = requestMsg.getMethodName();
           Vector args = requestMsg.getArgs();
           if ( args != null ) {
               Object argObjects[] = new Object[args.size()];
               for (int i=0; i<args.size(); i++) {
                   argObjects[i] = args.elementAt(i);
               }
               result = executeMethodWithArgs(methodName, argObjects);
               resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                       							   result);
               // we have to check this the "hard way" because Class.getMethod(name, args[])
               // does not work if the args are subclasses of the parameter classes

               // find the method matching the method name requested
               /*
               Method methods[] = this.getClass().getMethods();
               boolean foundMethod = false;
               for (int i=0; i<methods.length; i++) {
                   if ( methods[i].getName().equals(methodName) ) {
                       // check to see if the parameter classes match the arguments
                       Class parameters[] = methods[i].getParameterTypes();
                       boolean classesMatch = false;
                       if ( parameters.length == argObjects.length) {
                           classesMatch = true ;
                           for (int j=0; j<parameters.length; j++) {
                               if (!parameters[j].isInstance(argObjects[j])) {
                                   classesMatch = false;
                                   break;
                               }
                           }
                       }
                       // if we found the right method, execute it
                       if ( classesMatch ) {
                           foundMethod = true;
                           result = methods[i].invoke(this, argObjects);
                           resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                                                               result);
                           break;
                       }
                   }
               }
               // if we didn't find the method, throw an exception
               if ( ! foundMethod ) {                    
                   throw new NoSuchMethodException(methodName);
               }
               */
           }
           // args == null, invoke method with no args
           else {
               result = this.getClass().getMethod(methodName, null).invoke(this, null);
               resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                                                   result);
           }
       }
       catch (Exception xcp) {
           if ( xcp instanceof java.lang.reflect.InvocationTargetException ) {
               java.lang.reflect.InvocationTargetException inex = (java.lang.reflect.InvocationTargetException) xcp;
               resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                                               inex.getTargetException());
           }
           else {
               resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                                               xcp);
           }
           if (_logger.isEnabledFor(Logger.ERROR))
               _logger.logMessage("", xcp, Logger.ERROR);              
       }
       return resultMsg;
   }
   
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
