/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/16 21:27:25 $
 */

/** 
 * This is a test agent simmulating users loggin to the system and enforcing different policies on them
 */
package kaos.core.csi.usecase;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.Transport;
import kaos.core.csi.TransportImpl;
import kaos.core.csi.KAoSActorImpl;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;

import kaos.core.csi.extension.history.logging.HistoryMonitor;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.csi.policy.ObligationViolationException;
import kaos.core.csi.policy.ViolatedObligationDescription;
import kaos.core.csi.query.QueryState;

import kaos.ontology.vocabulary.ActionStatusConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.ontology.vocabulary.computing.ComputingActionConcepts;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Set;

import java.io.InputStreamReader;
import java.io.LineNumberReader;

/**
 * $Revision: 1.7 $
 */
public class LoginSimulationAgent extends KAoSActorImpl
{
	public LoginSimulationAgent(String guid, String name, Vector domainNames, Transport transport) throws Exception
	{
        // Nothing special here
		super(guid, name, domainNames, transport);	// This will register an agent in KAoS
        CSIFactory.setCurrentTransport(transport.getName());
	}

	public static void main(String [] args) throws Exception      
	{
        // -- Begin -- Login Agent Initilization
        
		LoginSimulationAgent logginAgent;
		String logginAgentName = "LoginSimulationAgent"; // defult agent name
		String transport = CSIFactory.TCP_TRANSPORT;     // defulat communitation protocol     
		Vector domainNames = new Vector();
		domainNames.add("LoginSystemDomain");            // defult domain for the agent
        	    
		Transport trans = new TransportImpl();
		trans.setName(transport);
        
        logginAgent = new LoginSimulationAgent(logginAgentName,    	//guid
                                               logginAgentName,    	//nickname
                                               domainNames,   		//domain names
										       trans);  			//agent transport
        
        // Add actor to an ontology class
        logginAgent.addOntologicalType(ActorConcepts.SoftwareActor(), null);
        
        logginAgent.registerWithKAoS();
	
        //-- End of Login Agent Initialization
        //--------------------------------------------------------------------------------------------------
        
		int noOfFailureForGivenUsername = 0;

        // -- Obtain the list of known humans
		
		QueryState queryState = CSIFactory.getQueryState();
		PolicyChecking policyChecking = CSIFactory.getPolicyChecking();
		HistoryMonitor historyMonitor = CSIFactory.getHistoryMonitor();
		
		Set humans = queryState.getInstancesOf(ActorConcepts.Human());
        
        Hashtable userNames = new Hashtable();       // keeps mapping from short name (username) to the full (url) name of the human
		Hashtable logins = new Hashtable();          // keeps mapping from short name (username) to password
        Hashtable nameNoOfFailure = new Hashtable(); // maps username to the current number of logging failure for a given username
        
		if (humans != null)
		{			
			System.out.println("\n Available logins: [name, password]\n");
			
            int i = 0; // needed to build unique password for each username
			for (Iterator it = humans.iterator(); it.hasNext(); )
			{
				String currentHuman = (String)it.next();
                String currentUsername = currentHuman.substring((currentHuman.indexOf("#") + 1));
				System.out.println("\n " + currentUsername + " pwd" + i);
                
                // put information about this human into the mapping tables
                userNames.put(currentUsername, currentHuman);
				logins.put(currentUsername, "pwd"+i);
                nameNoOfFailure.put(currentUsername, new Integer(noOfFailureForGivenUsername));
				i++;
			}
		}
		else
		{
			System.out.println("\n No humans found");
			System.exit(0);
		}				
		// End -- Obtain the list of known humans
        
		boolean loggedIn = false; // Status of the loggin agent - indicates if somebody is currently logged in
        
		InputStreamReader console = new InputStreamReader(System.in);	
		LineNumberReader lnr = new LineNumberReader(console);

        int logginAIDno = 0; // needed to build unique id for each action instance descriptions
        
		while(true) // Wait for a user input
		{								
			String providedUsername;
			String providedPassword;

			if (loggedIn)
			{
				System.out.println("\n please enter 'logout' to log out.\n ");
                providedUsername = lnr.readLine();

				if ( !providedUsername.equals("logout"))
					continue;

				loggedIn = false;
			}

            // Ask for username
			System.out.println("\n enter login name: ");	
            System.out.flush();
            providedUsername = lnr.readLine();
			providedUsername.trim();			

            // Find if the provided user name is known
            String currentUserPassword = (String)logins.get(providedUsername);
			if (currentUserPassword == null)
			{
				System.out.println("\n Given username is not known!");
				continue;
			}

            // Ask for password
			System.out.println("\n enter password: ");
            System.out.flush();
            providedPassword = lnr.readLine();
            providedPassword.trim();
		
            // --------------- Build Action Instance Description for the LoginAction
            
            ActionInstanceDescription logginAID = 
                new ActionInstanceDescriptionImpl("logginAID" + logginAIDno, 
                		                          ComputingActionConcepts.LoginAction(),
                		                          logginAgentName);
            logginAIDno++; // increase the counter of constructed AIDs
            
			OntPropertyDescription hasUsernameProperty = new OntPropertyDescriptionImpl(ComputingActionConcepts.hasUsername());	
            hasUsernameProperty.addValue((String)userNames.get(providedUsername));
            logginAID.addProperty(hasUsernameProperty);
	
            OntPropertyDescription hasPasswordProperty = new OntPropertyDescriptionImpl(ComputingActionConcepts.hasPassword());	
            hasPasswordProperty.addValue(providedPassword);
            logginAID.addProperty(hasPasswordProperty);	
            	
			try
			{
				policyChecking.findPolicyDecision(logginAID, null);
			}
			catch (KAoSSecurityException e)
			{
				System.out.println("\n Policy forbids this login action!");
				continue;
			}
			catch(ObligationViolationException ove){
            	if (  ove.getViolatedObligations() != null ){
            		List violatedobligations = ove.getViolatedObligations();
            		for (Iterator it = violatedobligations.iterator(); it.hasNext(); )
        			{
        				ViolatedObligationDescription violationDesc = (ViolatedObligationDescription)it.next();
        				System.out.println("\n\t****	Violated obligation:\n\t" + violationDesc);
        				System.out.println("\n\t****	Reason for violation:\n\t" + violationDesc.getReasonForViolation());              		
        			}
            	}
            	continue;
            	
            }

			if (providedPassword.equals(currentUserPassword))
			{				
				System.out.println("\n Logged in to the system");				
				loggedIn = true;
			}
			else
			{
				System.out.println("\n Incorrect password! login failed!");	
                
                // Add the status information to the current Action Instance Description of the Logging Action
                OntPropertyDescriptionImpl hasStatusProperty =  new OntPropertyDescriptionImpl(ActionStatusConcepts.hasStatus());
                hasStatusProperty.addValue(ActionStatusConcepts.DefaultFailureActionStatus);
                logginAID.addProperty(hasStatusProperty);
                
                // TODO: move this call to CSI Interface when it will be available in CSI
                historyMonitor.logEvent(logginAID);  

                try {
                	policyChecking.findPolicyDecision(logginAID, null);
                }
                catch(ObligationViolationException ove){
                	if (  ove.getViolatedObligations() != null ){
                		List violatedobligations = ove.getViolatedObligations();
                		for (Iterator it = violatedobligations.iterator(); it.hasNext(); )
            			{
            				ViolatedObligationDescription violationDesc = (ViolatedObligationDescription)it.next();
            				System.out.println("\n\t****	Violated obligation:\n\t" + violationDesc);
            				System.out.println("\n\t****	Reason for violation:\n\t" + violationDesc.getReasonForViolation());             		
            			}
                	}
                	
                }
                int newNoOfFailure = ((Integer)nameNoOfFailure.get(providedUsername)).intValue() + 1;
				System.out.println("\n # Failed LOGIN attempts so far for this username: " + newNoOfFailure);
                
                // Increase the counter of failure for a given user
                nameNoOfFailure.put(providedUsername, new Integer(newNoOfFailure++));
			}									
		}
	}
}