/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 15:32:56 $
 */
package kaos.core.csi.usecase;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.query.QueryRegistration;
import kaos.core.csi.registration.Registration;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.util.MsgUtils;

import javax.agent.TransportMessage;
import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.transport.MessageListener;
import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;
import javax.agent.service.directory.SearchException;

import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * $Revision: 1.5 $
 */
public class SimpleGuardedSender
    implements  MessageListener
    
 {   
    /**
        This class llustrates an implementation of a SimpleGuardedSender: an entity registered with KAoS via its Guard
        and capable of sending messages to other entities registered in the KAoS Directory Service.  Each message send is authorized according to the current set of policies
        stored by the Guard.
        
        Configuration data includes: nickname, the name of the entity to send a message to, names of kaos domains it will 
        register in, name of a transport to use for communication within KAoS.
        
        Bootstrapping steps:
	        - obtains a reference to CSI Registration
	        - binds itself to CSI Transport 
	        - asks CSI Registration  to create a KAoSAgentDescription for  itself
	        - asks CSI Registration to register with KAoS

        Running
	        - looks up another agent, whose name is given
	        - sends a message to it
    */
    protected KAoSAgentDescription      _agentDescription = null;
    protected TransportSupport          _transportSupport;
    protected QueryRegistration         _query; 
    private Hashtable                   _receiverDescriptions;
    
    public SimpleGuardedSender( )
    {
    }

    //////////////////////////////////////////////////////////////

    public void initialize(String name, KAoSAgentDescription agentDescription, String transportName)
    throws Exception
    {
        CSIFactory.setCurrentTransport(transportName);
        Registration registration = CSIFactory.getRegistration();
        _agentDescription = agentDescription;
                                                                    
        _transportSupport = CSIFactory.getTransportSupport();
        
        _transportSupport.bindMsgListenerToTransport(this, name, agentDescription);
        
        _query = CSIFactory.getQueryRegistration();
        
        registration.registerEntity(_agentDescription,
                                    true,       // guarded
                                    false);     // replace registration 
        _receiverDescriptions = new Hashtable(); 
       
    }
    
    //////////////////////////////////////////////////////////////
    // Implementation of the MessageListener interface

    /**
     * Implements the receiveMessage method mandated by the
     * MessageListener interface.  This method prints
     * the incoming TransportMessage to System.out.
     *
     */
    public void receiveMessage( TransportMessage tmsg )
    {
        System.out.println( "\n   *** SimpleGuardedSender: received transport message:\n" + tmsg);
        Object msgContent = MsgUtils.unwrapObjectContent(tmsg);

        if ( msgContent == null ) {
            System.out.println("\n   *** SimpleGuardedSender: \n receiveMessage: could not retrieve the content from: " + tmsg);
            return;
        }
                    
       String senderName = MsgUtils.getSender(tmsg);
       
       System.out.println("\n   *** SimpleGuardedSender: received the following content from " + senderName + ": " + msgContent);
    }

    /**
     * Two agents are equal if their respective hash codes
     * are equal.
     *
     * @return true if obj equals this SimpleGuardedSender.
     */
    public boolean equals( Object obj )
    {
        if ( obj == this )
            return true;

        if ( !(obj instanceof SimpleGuardedSender) )
            return false;

        return this.hashCode() == ((SimpleGuardedSender)obj).hashCode();
    }

    /**
     * Determines the hash code for the SimpleGuardedSender.  The SimpleGuardedSender
     * hash code is simply the hash code of the represented agents
     * AgentName.  If the AgentName field has not yet been set,
     * then the hash code is found from <code>super.hashCode()
     * </code>.
     *
     * @return int the hashCode for this SimpleGuardedSender.
     *
     */
    public int hashCode()
    {
        return _agentDescription.getEntityNameAsString() == null ? super.hashCode() : _agentDescription.getEntityNameAsString().hashCode();
    }

    //////////////////////////////////////////////////////////////
    
    public void sendMessageTo(String receiverName)
        throws DirectoryFailure, SearchException, NotLocatableException, TransportFailure
    {
        // First, lookup the receiver with the given receiverName.
        KAoSAgentDescription receiverDesc = null;
        if ( _receiverDescriptions.containsKey(receiverName)) {
            receiverDesc = (KAoSAgentDescription) _receiverDescriptions.get(receiverName);
        }
        else {
            receiverDesc = _query.getActor(receiverName);
            _receiverDescriptions.put(receiverName, receiverDesc);
        }
        
        System.out.println("\n   *** SimpleGuardedSender: Requested description for: " +  receiverName 
                          +"\n                  Received: \n\n    " + receiverDesc);
        if ( receiverDesc == null ) return;
        
        // Then ask the transport support service to send a messager to the receiver.
        _transportSupport.sendMessageTo(_agentDescription,
                                        receiverDesc,
                                        "\n\n   Hello World!");
       
       System.out.println("\n   *** SimpleGuardedSender: Sent the following message to " +  receiverName + ": \n\n   Hello World!");
    }
    
   //////////////////////////////////////////////////////////////
   // Entry point...

   public static void main(String [] args)
      throws Exception      
    {
        String sender = null;
        String transport = null;
        String receiver = null;
        Vector domainNames = null;
        if (args.length > 0) {
	        int i = 0;
	        while (i < args.length) {
	            if (args[i].equalsIgnoreCase ("-sender")) {
	                if ((i+1) < args.length) {
    	                sender = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify Entity Name with -sender");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-receiver")) {
	                if ((i+1) < args.length) {
    	                receiver = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-transport")) {
	                if ((i+1) < args.length) {
    	                transport = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-domains")) {
                    if ((i+1) < args.length) {
                        String domainsStr = args[i+1];
                        domainNames = new Vector();
                        StringTokenizer tokenizer = new StringTokenizer(domainsStr, ",");
                        while (tokenizer.hasMoreTokens()) {
                            domainNames.add(tokenizer.nextToken());
                        }
                    }
                }// else if
                 i++;
	        }// while
	    }
	    
        if ( sender == null) {
            System.out.println ("\n     *** Must specify Sender Name with -sender\n");
            return;
        }
        
        if ( receiver == null) {
            System.out.println ("\n     *** Must specify Receiver Name with -receiver\n");
            return;
        }
        
        if ( transport == null) {
            System.out.println ("\n     *** Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
            return;
        }
        
        if ( domainNames == null ) {
            domainNames = new Vector();
            domainNames.add("SenderDomain");
            System.out.println (" <>    Warning: No Domain Names specified with -domains, using default: " + domainNames);
        }
        
        
        System.out.println ("\n     <>  Sender name                 " + sender);
        System.out.println ("\n     <>  Receiver name               " + receiver);
	    System.out.println ("\n     <>  Requested transport:        " + transport);
	    System.out.println ("\n     <>  Requested domains:          " + domainNames + "\n");
	    
        
        Registration registration = null;
        try {
            CSIFactory.setCurrentTransport(transport);
            registration = CSIFactory.getRegistration();
        }
        catch(Exception e){
            e.printStackTrace();
            System.out.println ("  **** Unable to obtain CSI services: Registration. Exiting...");
            return;
        }
        
        Enumeration domainsEnum = domainNames.elements();
        while ( domainsEnum.hasMoreElements() ) {
            String domainName  = (String) domainsEnum.nextElement();
            System.out.println("\n     <>  Will register domain: " + domainName + "\n");
            
            try {
                registration.registerDomain(domainName);
            }
            catch(DirectoryFailure de) {
                System.out.println("\n   Due to Directory Failure unable to register domain: " + domainName);
                de.printStackTrace();
                return;
            }
            catch(AlreadyRegisteredException ae) {
                System.out.println("    The following domain has already been registered: " + domainName);
            }
        }
        
        // Ask Registration to create a KAoSAgentDescription for the sender, so it can bind to the transport.
        KAoSAgentDescription agentDescription = registration.createKAoSAgentDescription(sender,       // use nickname as guid
                                                                                        sender,       // nickname
                                                                                        domainNames,  // domains to register in
                                                                                        transport,
                                                                                        null);      // no extra transport info necessary
                                                                    
        
        SimpleGuardedSender usecase = new SimpleGuardedSender();
        
        try {
            usecase.initialize(sender, agentDescription, transport);
        }
        catch(Exception e) {
            e.printStackTrace();
            return;
        }
       
       while (true) {
            usecase.sendMessageTo(receiver);
            try {
                Thread.currentThread().sleep(5000);
            }
            catch(Exception e){
                e.printStackTrace();
                break;
            }
       }
       
        synchronized ( Thread.currentThread() ) {
         Thread.currentThread().wait();

      }

   }
}

