/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/12/16 16:35:56 $ 
 */
package kaos.core.csi.usecase.legacy;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.StringTokenizer;

import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;

import kaos.core.csi.KAoSActor;
import kaos.core.csi.KAoSActorImpl;
import kaos.core.csi.Transport;
import kaos.core.csi.TransportImpl;
import kaos.core.csi.registration.Property;
import kaos.core.csi.usecase.legacy.vocabulary.LegacySystemActionConcepts;
import kaos.core.util.KAoSConstants;

/**
 * 
 * @author KAoS Team
 *
 */
public class KAoSWrappedLegacySystem 
	extends LegacySystemImpl 
	implements KAoSActor
{
	private static  LegacySystemEnforcerImpl     		__INSTANCE;
    private static boolean                      		__INITIALIZED = false;
	protected KAoSActorImpl								_kaosActor;
	
	
	/**
     * Retrieve a reference to the LegacySystem.
     *
	 * @param       args   	are the command line args for the LegacySystemEnforcerImpl.
	 * 						They are:
	 * 						 -transport tcp -domains LegacyDomain
	 *                                      
     * @return a reference to the RequestManager
     */
    public static LegacySystem getInstance(String [] args)
        throws Exception
    {
    	 if ( __INITIALIZED ) return __INSTANCE;
         
        initialize(args);
        
         __INITIALIZED = true;
         return __INSTANCE;
    }
    
    private static void initialize(String [] args)
    throws Exception
    {  
    	__INSTANCE = new LegacySystemEnforcerImpl("LegacySystem", LegacySystemActionConcepts.LegacySystemAction());
	
    	KAoSWrappedLegacySystem legacySystem = new KAoSWrappedLegacySystem();
    	legacySystem.init(args);
    	__INSTANCE.setControlledLegacySystem(legacySystem);
    	
    }
    
	protected KAoSWrappedLegacySystem() {
		super();
	}
	
	private void init(String [] args)
	throws Exception
	{
		// Retrieve from args only transport and domain names for KAoSActor.  
		// The name is "LegacySystem".
		String transport = null;
		Vector domainNames = null; 
		if (args.length > 0) {
			int i = 0;
			while (i < args.length) {
				if (args[i].equalsIgnoreCase ("-transport")) {
  	          		if ((i+1) < args.length) {
  	          			transport = args[i+1];
  	          			i++;
  	          		}
  	          		else {
  	          			System.out.println ("\n	**** Must specify Agent transport with -transport");
  	          			throw new Exception();
  	          		}
				}
				else if (args[i].equalsIgnoreCase ("-domains")) {
					if ((i+1) < args.length) {
						String domainsStr = args[i+1];
						domainNames = new Vector();
						StringTokenizer tokenizer = new StringTokenizer(domainsStr, ",");
						while (tokenizer.hasMoreTokens()) {
							domainNames.add(tokenizer.nextToken());
						}
					}
				}// else if
				i++;
			}// end while
		}
         
		if ( domainNames == null ) {
			domainNames = new Vector();
			// try the following property, if not specified on the command line
			String domainNamesStr = System.getProperty(KAoSConstants.DOMAIN_NAMES);
			if ( domainNamesStr != null ) {  		 
				StringTokenizer st = new StringTokenizer (domainNamesStr, ",");
				while (st.hasMoreTokens()) {
					String str = st.nextToken();
					domainNames.addElement (str);
				}
			}
		} 
         
		if (domainNames.isEmpty()) {
			domainNames.add("TestDomain");
			System.out.println (" <>    Warning: No Domain Names specified, using default: " + domainNames);
		}
  	   
		Transport trans = new TransportImpl();
		trans.setName(transport);
        
		_kaosActor = new KAoSActorImpl(null,  			// guid
                					  "LegacySystem", 	// name
                					  domainNames,
                					  trans);
		_kaosActor.registerWithKAoS();  
	}
	
	////////////////////////////////// KAoSActor interface implementation
	public String getConceptMappingName() {
		return _kaosActor.getConceptMappingName();
	}

	public List getDomainNames() {
		return _kaosActor.getDomainNames();
	}

	public String getGUID() {
		return _kaosActor.getGUID();
	}

	public String getName() {
		return _kaosActor.getName();
	}

	public Map getEntityOntologicalTypes() {
		return _kaosActor.getEntityOntologicalTypes();
	}

	public Enumeration getEntityOntologicalTypeNames() {
		return _kaosActor.getEntityOntologicalTypeNames();
	}

	public boolean isGuarded() {
		return _kaosActor.isGuarded();
	}

	public boolean isRegistered() {
		return _kaosActor.isRegistered();
	}

	public void setRegistered(boolean registered) {
		_kaosActor.setRegistered(registered);

	}

	public List getCapabilities() {
		return _kaosActor.getCapabilities();
	}

	public Hashtable getProperties() {
		return _kaosActor.getProperties();
	}

	public Property getProperty(String name) {
		return _kaosActor.getProperty(name);
	}

	public Object getPropertyValue(Property property) {
		return _kaosActor.getPropertyValue(property);
	}

	public void sendMessage(Serializable messageContent, KAoSActor receiver)
			throws NotLocatableException, TransportFailure {
		_kaosActor.sendMessage(messageContent,receiver) ;
	}

	public void receiveMessage(Serializable messageContent, KAoSActor sender) {
		System.out.println("\nReceived: " + messageContent + "\nfrom: " + sender);
	}

	public List getTransports() {
		return _kaosActor.getTransports();
	}

	public Transport getTransport(String name) {
		return _kaosActor.getTransport(name);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
