/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/03 18:47:05 $
 */
package kaos.core.csi.usecase.legacy;


import java.util.Vector;

import kaos.core.csi.KAoSActorImpl;
import kaos.core.csi.Transport;
import kaos.core.csi.usecase.legacy.vocabulary.LegacySystemActorConcepts;

/**
 * This class illustrates how a legacy client becomes KAoS-aware by simply inheriting
 * from the KAoSActorImpl class, adding the "LegacySystemActorConcepts.LegacySystemUser()" ontological type 
 * to its KAoS description and registering with KAoS (all shown in the start() method).
 * 
 * @author KAoS Team
 * 
 * $Revision: 1.3 $
 */
public class LegacySystemClient extends KAoSActorImpl {

	/**
	 * Instantiates a new KAoS-aware LegacySystemClient using KAoS services for transport, naming, 
     * agent directory and policy enforcement.
     * 
	 * @param guid			String containing the unique id of the client.
	 * @param name			String containing a nickname of the client.
	 * @param domainNames	Vector containing the names of KAoS domains the client will register in.
	 * @param transport		String containing the name of the transport to use in communication.
	 * 
	 * @throws Exception	to be thrown when connection to KAoS services cannot be established.
	 * 
	 */
	public LegacySystemClient(String guid, String name, Vector domainNames,
			Transport transport) throws Exception {
		super(guid, name, domainNames, transport);
	}

	/**
	 * Instantiates a new KAoS-aware LegacySystemClient using KAoS services for transport, naming, 
     * agent directory and policy enforcement.
     * 
	 * @param args			command line arguments containing the client's name, transport name and names
	 * 						of domains, in which the client will register.  Example arguments:
	 * 						-name LegacyClient1 -transport tcp -domains LegacyDomain 
	 * 
	 * @throws Exception	to be thrown when connection to KAoS services cannot be established.
	 * 
	 */
	public LegacySystemClient(String[] args) throws Exception {
		super(args);
		
	}
	
	/**
	 * Add the "LegacySystemUser" ontological type to the description of this client that
	 * will be stored in the KAoS Directory Service.
	 * Register this client with the KAoS Directory Service.
	 * 
	 * @throws Exception 	if the connection to the KAoS Directory Service cannot be established
	 * 						due to transport failure, or the Directory Service fails.
	 */
	public void start()
	throws Exception
	{
    	_actorDesc.setEntityOntologicalType(LegacySystemActorConcepts.LegacySystemUser());	
    	
    	registerWithKAoS();
	}
	
	public void requestActionFromLegacySystem(LegacySystem ss)
	{	
		ss.legacyAction(getGUID());
	}

	/**
	 * Obtain a reference to the LegacySystem from the LegacySystemFactory.  
	 * 
	 * Instantiate a LegacySystemClient and make it request a legacy action from the LegacySystem.
	 * 
	 * @param args 			command line arguments containing the client's name, transport name and names
	 * 						of domains, in which the client will register.  Example arguments:
	 * 						-name LegacyClient1 -transport tcp -domains LegacyDomain 
	 * 
	 */
	public static void main(String[] args)
	throws Exception
	{
		
		LegacySystem ls = LegacySystemFactory.getLegacySystem(args);
		LegacySystemClient client = new LegacySystemClient(args);
		client.start();
		
		Thread.sleep(4000);
		
		client.requestActionFromLegacySystem(ls);
		
		synchronized ( Thread.currentThread() ) {
	         Thread.currentThread().wait();
	      }

	}

}
