/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/03 18:47:05 $
 */

package kaos.core.csi.usecase.legacy;


public class LegacySystemFactory {

	/**
	 * This is a factory class returning a reference to an implementation of the LegacySystem.  Clients 
	 * of LegacySystems are expected to ask this factory for instances of LegacySystems.  
	 * The factory hides from the clients whether the returned LegacySystem is KAoS-aware.
	 * In the example below if the property "use.old.legacy.system" is present, then the old Legacy System
	 * will be returned, otherwise, KAoS-aware Legacy System is returned. However, the client's code remains 
	 * unchanged and the client will be totally unaware that the LegacySystem it accesses uses KAoS services.
	 * 
     * Retrieve a reference to the LegacySystem.
     *                                   
	 * @param       args	command line arguments containing the client's name, transport name for communication
	 * 						with the KAoS Directory Services and domain names, in which the client will be registered.
	 * 						Example arguments:
	 * 						-name LegacyClient1 -transport tcp -domains LegacyDomain 
     *
     * @return a reference to the LegacySystem
     */
	public static LegacySystem getLegacySystem(String [] args) throws Exception
    {   
		if ( System.getProperty("use.old.legacy.system") != null  ){
			return new LegacySystemImpl();
		}
		else {
			return KAoSWrappedLegacySystem.getInstance(args);
		}
    
    }
}
