/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/26 00:39:25 $
 */

package kaos.core.csi.util.tcpSupport;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;

import kaos.core.util.Logger; 

/**
 * This class is a TCP server that listens for clients and 
 * provides them access to KAoS services through the CSI 
 * interfaces using a KAoS CSI TCP protocol
 * 
 */
public class KAoSCSIServer 
{
    protected ServerSocket              _serverSocket = null;
	protected ServerRequestHandler      _serverRequestHandler;
    private Logger                      _logger = Logger.getLogger(this);

	public KAoSCSIServer()
		throws IOException
	{
		init(0);
	}
	
	public KAoSCSIServer(int port)
		throws IOException
	{
		init(port);
	}
	
    /**
     * Tries to start a socketserver on the specified port
     * and start a thread to listen for clients
     * 
     * @param	iPort
     */
    public void init (int iPort)
		throws IOException
    {
		if (_logger.isEnabledFor(Logger.DEBUG)){
			_logger.logMessage(".init: trying to create a comm server on port = " + iPort, Logger.DEBUG);
		}
		_serverSocket = new ServerSocket (iPort);
		if (_logger.isEnabledFor(Logger.DEBUG)){
			_logger.logMessage(".init: created comm server on port = " + iPort, Logger.DEBUG);
		}     
		Thread svrThrd = new Thread (new ServerThread());
        svrThrd.setDaemon (true);
        svrThrd.start();
    }    

	//public KAoSCSIClientHandler getMostRecentClientHandler()
	public ServerRequestHandler getMostRecentRequestHandler()
	{
		return _serverRequestHandler;
	}

	//Server Thread ///////////////////////////////////////////////////////////
    /**
     * Listens for new connections and spawns a handler for each connection
     */
    public class ServerThread implements Runnable
    {
        public void run()
        {
            while (_serverSocket != null && !_serverSocket.isClosed()) {
                Socket clientSocket = null;
                try {
                  
					//listen for clients
					if (_logger.isEnabledFor(Logger.INFO)){
						_logger.logMessage(".ServerThread: Waiting for next client", Logger.INFO);
					}
                     clientSocket = _serverSocket.accept();
					//create sender to pass info to client
					if (_logger.isEnabledFor(Logger.INFO)){
						_logger.logMessage(".ServerThread: creating client handler", Logger.INFO);
					}
					//_serverRequestHandler = new ServerRequestHandler(clientSocket);
					ServerRequestHandler requestHandler = new ServerRequestHandler(clientSocket);
					if (_logger.isEnabledFor(Logger.DEBUG)){
						_logger.logMessage(".ServerThread: created a new ServerRequestHandler for remote port:"
								           +  + clientSocket.getPort(), Logger.DEBUG);
					}
	
					requestHandler.start();
                }
                catch (Exception se) {			
					se.printStackTrace();
					System.out.println("\n  *** ServerThread.run: unable to establish server handler for socket from: "
					                   + clientSocket.getInetAddress());
                }
            }// end while
			if (_logger.isEnabledFor(Logger.DEBUG)){
				_logger.logMessage(".ServerThread: ServerAccept Thread has Stopped.", Logger.DEBUG);
			}
        }
    }
	
	// test driver ///////////////////////////////////////////////////////////////
    public static void main (String[] args)
    {
		//init comm server to port specified by user otherwise use any available
        int port = 0;
        if (args.length > 0) {
            port = Integer.parseInt (args[0]);
        }
		try{
			KAoSCSIServer cs = new KAoSCSIServer(port);
		}
		catch(Exception e){
			e.printStackTrace();
			System.exit(0);
		}
		System.out.println("comm server listening on " + port);
		
		try{
			synchronized ( Thread.currentThread() ) {
				Thread.currentThread().wait();
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
    }
}