/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/27 22:04:00 $
 */
package kaos.core.csi.util.xmlSupport;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.CSIFactory;
import kaos.core.csi.OntPropertyDescriptionImpl;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.csi.registration.Registration;
import kaos.core.csi.request.ActionFailureException;
import kaos.core.csi.request.RequestManager;
import kaos.core.csi.request.FeedbackReceiver;
import kaos.core.csi.util.tcpSupport.ResponseSender;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.util.Logger;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.computing.ComputingActionConcepts;
import kaos.policy.guard.GuardInstantiationException;
import kaos.spatial.Location3D;
import kaos.spatial.Orientation3D;

// !!!TO_DO: to resolve the following robots imports
import kaos.robots.util.Coordinate;
import kaos.robots.ontology.RobotActionConcepts;
import kaos.robots.util.Pose;

import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;
import javax.agent.service.ServiceFailure;



import java.util.Calendar;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Enumeration;
import java.util.Vector;

/**
 */
public class XMLToCSILink
implements FeedbackReceiver  {
    
    public  static final String                 DOMAIN_FOR_ALL_TCP_BASED_PROTOCOL = "TCP-TO-KAoS-Protocol";
    private static final XMLToCSILink           __INSTANCE = new XMLToCSILink() ;
    private static boolean                      __INITIALIZED = false;
    
    private boolean								_policyCheckingInitialized = false;
    private PolicyChecking						_policyChecking;
    private Registration                        _registration;   
    private RequestManager                      _requestMgr;
    private String                              _transportName;
	private Logger                              _logger = Logger.getLogger(this);
    private boolean                             _tcpDomainRegistered = false;        
    private Vector                              _domains; // all registering entities will belong to the same TCP__based domain
    private Hashtable                           _responseSenders;
    
    /**
     * Retrieve an instance of XMLToCSILink.
     *
     * @param transportName String specifying the name of the transport to use for communication with the KAoS Directory Service.
     * 
     * @return a singleton, XMLToCSILink.
     */
    public static XMLToCSILink getInstance(String transportName)
    throws Exception
    {
        if ( __INITIALIZED ) return __INSTANCE;
        
        __INSTANCE.initialize(transportName);
        __INITIALIZED = true;
        return __INSTANCE;
    }

    private XMLToCSILink()
    {
    }
    /////////////////////////////   PUBLIC METHODS BEGIN
    
    public void requestAction(ResponseSender responseSender, String xmlString, String xmlTransmissionType)
    {
        Hashtable actions = XMLToCSITranslator.translateActions(xmlString, xmlTransmissionType);
        if ( _logger.isEnabledFor(Logger.INFO) )
        	_logger.logMessage("requestAction: actions hashtable:\n" + actions, Logger.INFO);
        if ( actions == null ) return;
        try {
            if ( actions.containsKey(XMLToCSIConstants.COMMANDS)){
                requestCommand(responseSender, actions);
            }
            else if ( actions.containsKey(XMLToCSIConstants.REGISTRATIONS)){
                register(actions);
            }
            else if ( actions.containsKey(XMLToCSIConstants.AUTHORIZATIONS) ) {
            	Hashtable authorizations = (Hashtable)actions.get(XMLToCSIConstants.AUTHORIZATIONS);
            	System.out.println("\n\n\t*** XMLToCSILink:  will process AUTHORIZATIONS!");
            	authorizeActions(authorizations);
            }
            	
        }
        catch(Exception e){
            e.printStackTrace();
        }
        
    }
    
    public void authorizeActions(Hashtable authorizations)
    {
       Enumeration names = authorizations.keys();
       while ( names.hasMoreElements() ) {// main loop to retrieve all actions to be authorized
           String name = (String) names.nextElement();
           
           Hashtable actionHash = (Hashtable)authorizations.get(name);
           String actorName = (String) actionHash.get(XMLToCSIConstants.ACTOR);
           
           if ( actorName == null ) {
        	   if ( _logger.isEnabledFor(Logger.WARN))
        		   _logger.logMessage("authorizeActions: action: " + name + " has no actor.  Authorization will not be performed!", Logger.WARN);
        	   continue;
           }
           Hashtable properties = (Hashtable) actionHash.get(XMLToCSIConstants.PROPERTIES);
           
           ActionInstanceDescription actionAID = null;
           
           try{
        	   actionAID  =   new ActionInstanceDescriptionImpl(null, 
            		   							 				name,
            		   							 				actorName);
           }
           catch(Exception e){
        	   e.printStackTrace();
        	   if ( _logger.isEnabledFor(Logger.WARN))
        		   _logger.logMessage("authorizeActions: action: " 
        				              + name + " could not be translated into the required ActionInstanceDescription.  Authorization will not be performed!", Logger.WARN);
        	   continue;      	   
           }
           
          if ( properties != null &&  properties.size() > 0 ) {
        	  // create OntPropertyDesc for each property
        	  Enumeration propertyNames = properties.keys();
        	  while ( propertyNames.hasMoreElements() ) {// main loop to retrieve all properties
                  String propertyName = (String) propertyNames.nextElement();
                  String propertyValue = (String) properties.get(propertyName);
                  try {
                	  OntPropertyDescriptionImpl desc = new OntPropertyDescriptionImpl(propertyName);
                	  desc.addValue(propertyValue);
                	  actionAID.addProperty(desc);
                  }
                  catch(Exception e){
                	  e.printStackTrace();
                	  if ( _logger.isEnabledFor(Logger.WARN))
               		   _logger.logMessage("authorizeActions: property: " 
               				              + propertyName + " could not be translated into the OntPropertyDescription and will not be included in the authorized action!", Logger.WARN);
               	      continue;
                  }
        	  } // end while     	  
          }
         
          if ( _logger.isEnabledFor(Logger.INFO))
        	  _logger.logMessage("authorizeActions: the following AID will be checked for authorization:\n\n" + actionAID,Logger.INFO);
          
          if ( ! _policyCheckingInitialized ) {   
        	  initializePolicyChecking();
          }
          
          if ( ! _policyCheckingInitialized ){ 
        	  if ( _logger.isEnabledFor(Logger.WARN))
            	  _logger.logMessage("authorizeActions: unable to access policy service. Permission for the following AID will NOT be checked:\n\n" 
            			             + actionAID,Logger.WARN);
        	  return;        
          }
          
          try{
        	  _policyChecking.checkPermission(actionAID, null);
          }
          catch (KAoSSecurityException e){
				System.out.println("\n\n\t*** AUTHORIZATION FAILED!! FOLLOWING ACTION IS NOT AUTHORIZED!\n " + actionAID);
          }
          catch(ServiceFailure sf){
        	  sf.printStackTrace();				
          }                   
       }// end while ( names.hasMoreElements() 
    }
    public  void register(Hashtable registrations)
    throws DirectoryFailure,
           AlreadyRegisteredException,
           GuardInstantiationException
    {
        if ( ! _tcpDomainRegistered ) registerTCPDomain();
        
        Enumeration names = registrations.keys();
        while ( names.hasMoreElements() ) {// main loop to retrieve all entities to register
            String name = (String) names.nextElement();
            if ( name.equals( XMLToCSIConstants.REGISTRATIONS) ) continue;
            KAoSAgentDescription agentDescription = _registration.createKAoSAgentDescription(name,               // use nickname as guid
                                                                                            name,               // nickname
                                                                                            _domains,           // domains to register in
                                                                                            _transportName,     // DS transport
                                                                                            null);              // no extra transport info necessary
            /*
                Get the value associated with the entity name.  If the value is a String, it means
                that the registering entity has no spatial properties to register.  If the value
                is a Hashtable, it means that the entity has Location and Orientation, or Location only.
            */
            Object value = registrations.get(name);
            if ( value instanceof String ) {
                try {
                    _registration.registerEntity(agentDescription,
                                                 true,       // guarded
                                                 false);     // replace registration 
                    System.out.println ("\n  **** Registered Entity with desc:\n   " + agentDescription);
                }
                catch(Exception e){
                    e.printStackTrace();
                    if (_logger.isEnabledFor(Logger.WARN))
                        _logger.logMessage(" Registration of the following description failed:\n  " 
                                           + agentDescription,Logger.WARN);
                }
                continue;
            }
        
            if ( value instanceof Hashtable ) {
                Hashtable registrationInfo = (Hashtable) value;
                // get LOCATION, IF PRESENT
                Hashtable locationHash = (Hashtable) registrationInfo.get(XMLToCSIConstants.LOCATION);
                Location3D location = null;
                if ( locationHash != null ) {
                    String x = (String)locationHash.get(XMLToCSIConstants.XCOORDINATE);
                    String y = (String)locationHash.get(XMLToCSIConstants.YCOORDINATE);
                    String z = (String)locationHash.get(XMLToCSIConstants.ZCOORDINATE);
                    double dx;
                    double dy;
                    double dz;
                
                    // should we require all 3 coords?  From the Location3D constructor, it looks like it...
                    if ( x != null && y != null && z != null  ) {
                        try {
                            dx = Double.parseDouble(x);
                            dy = Double.parseDouble(y);
                            dz = Double.parseDouble(z);
                            location = new Location3D(name, dx, dy, dz);
                        }
                        catch(NumberFormatException e) {
                            e.printStackTrace();
                        }
                    } // end if ( x != null && y != null && z != null  )
                }// end if (locationHash != null ) 
                Orientation3D orientation = null;
                if ( location != null ) { // get orientation
                    Hashtable orientationHash = (Hashtable) registrationInfo.get(XMLToCSIConstants.ORIENTATION);
                    if ( orientationHash != null ) {
                        String roll = (String)orientationHash.get(XMLToCSIConstants.ROLL);
                        String pitch = (String)orientationHash.get(XMLToCSIConstants.PITCH);
                        String yaw = (String)orientationHash.get(XMLToCSIConstants.YAW);
                        double dx;
                        double dy;
                        double dz;
                
                        // should we require all 3 coords?  From the Location3D constructor, it looks like it...
                        if ( roll != null && pitch != null && yaw != null  ) {
                            try {
                                dx = Double.parseDouble(roll);
                                dy = Double.parseDouble(pitch);
                                dz = Double.parseDouble(yaw);
                                orientation = new Orientation3D(name, dx, dy, dz);
                            }
                            catch(NumberFormatException e) {
                                e.printStackTrace();
                            }
                        }// end if ( x != null && y != null && z != null  ) 
                    }// end if (orientationHash != null )
                    
                    try {
                        _registration.registerEntityWithSpatialProperties(agentDescription,
                                                                          location, 
                                                                          orientation,
                                                                          null, // occupied space (block)
                                                                          true,  // guarded
                                                                          false); // replace registration
                        System.out.println ("  **** Registered EntityWithSpatialProperties with desc:\n   " + agentDescription
                                + "\n   location: " + location + "\n    orientation: " + orientation);
                    }
                    catch(Exception e)
                    {
                        e.printStackTrace();
                        if (_logger.isEnabledFor(Logger.WARN))
                            _logger.logMessage(" Registration of the following description failed:\n  " 
                                               + agentDescription,Logger.WARN);
                    }
                    continue;
                }// end if ( location != null 
                
                try {
                    _registration.registerEntity(agentDescription,
                                                 true,       // guarded
                                                 false);     // replace registration 
                    System.out.println ("\n  **** Registered Entity with desc:\n   " + agentDescription);
                }
                catch(Exception e){
                    e.printStackTrace();
                    if (_logger.isEnabledFor(Logger.WARN))
                        _logger.logMessage(" Registration of the following description failed:\n  " 
                                           + agentDescription,Logger.WARN);
                }
                        
            }// end if ( value instanceof Hashtable )
        
        }// end while ( names.hasMoreElements() )
    }
    
     public void requestCommand(ResponseSender responseSender, Hashtable commands)
     {
        Enumeration names = commands.keys();
        while ( names.hasMoreElements() ) {// main loop to retrieve all command
            String name = (String) names.nextElement();
            
            if ( name.equals( XMLToCSIConstants.COMMANDS) ) continue;
            Hashtable commandHash = (Hashtable) commands.get(name);
            
            // Check for moveTo command.  An example of a Hashtable containing the MoveTo
            // info is:
            //
            // {Commands=Commands, MoveTo={movingTo={XCoordinate=7.0, YCoordinate=9.0}, Receiver=Ivan}}
            //
            if ( name.equals(XMLToCSIConstants.MOVE_TO)) {
                Hashtable movingTo = (Hashtable)commandHash.get(XMLToCSIConstants.MOVING_TO);
                String receiver = (String)commandHash.get(XMLToCSIConstants.RECEIVER);
               
                if ( movingTo != null && receiver != null ) {
                    String x = (String)movingTo.get(XMLToCSIConstants.XCOORDINATE);
                    String y = (String)movingTo.get(XMLToCSIConstants.YCOORDINATE);
                    double dx =0.0;
                    double dy = 0.0;
                    double dz = 0.0;
                
                    // should we require all 3 coords?  From the Location3D constructor, it looks like it...
                    if ( x != null && y != null  ) {
                        try {
                            dx = Double.parseDouble(x);
                            dy = Double.parseDouble(y);
                            }
                        catch(NumberFormatException e) {
                            e.printStackTrace();
                        }
                        HashMap properties = new HashMap();
                        Coordinate coord = new Coordinate(dx, dy, 0);
				        properties.put(ActionConcepts.movingTo(), coord);
				        try{
				          
					        String actionID = _requestMgr.sendHashMapRequest(this, 
					                                       receiver, 
					                                       RobotActionConcepts.MoveTo(), 
					                                       properties);
					       _responseSenders.put(actionID,responseSender);
				        }
				        catch(ActionFailureException ae){
					        ae.printStackTrace();	
					        responseSender.sendResponse ("S-FEEDBACK\nAction status: MoveTo ACTION FAILED!!\nFrom: " + receiver + "\nE-FEEDBACK");
				        }
				        catch(NotLocatableException nle){
					        nle.printStackTrace();	
					        responseSender.sendResponse ("S-FEEDBACK\nAction status: MoveTo ACTION FAILED!!\nReason:   Unable to locate and communicate with the request receiver: " 
					                                    + receiver + "\nE-FEEDBACK");
				        }
				        catch(TransportFailure tf){
					        tf.printStackTrace();	
					        responseSender.sendResponse ("S-FEEDBACK\nAction status: MoveTo ACTION FAILED!!\n   Unable to communicate with the request receiver: "
					                                     + receiver + "\n   Transport to Directory Service failed!\nE-FEEDBACK");
				        }
                    }
                }// end validating x,y: if ( movingTo != null && receiver != null )
            }// end if ( name.equals(XMLToCSIConstants.MOVE_TO))
            
            // Check for GetPose command.
            // An example of getPose command Hashtbale is:
            //
            //{Commands=Commands, GetPose={Receiver=Ivan}}
            //
            if ( name.equals(XMLToCSIConstants.GET_POSE)) {
                String receiver = (String)commandHash.get(XMLToCSIConstants.RECEIVER);
                if ( receiver != null ) {
                    try{
					    Object result = _requestMgr.sendHashMapRequest(receiver, 
					                                                   RobotActionConcepts.GetPose(), 
					                                                   null);
					    // this is a temporary hack to send the result back to the client
					    
					    String resultStr = getPoseAsString((Pose)result, receiver);
					    System.out.println(" XMLToCSILink: returning pose: \n" +  "Result \n" + resultStr);
					    responseSender.sendResponse("S-FEEDBACK\nResult \n" + resultStr + "\nE-FEEDBACK");
					 
		            }
				    catch(Exception e){
					   e.printStackTrace();				
				    }
                }
            }//if ( name.equals(XMLToCSIConstants.GET_POSE))
        }// end while ( names.hasMoreElements() 
     }
    ///////////////////////////////////////////////////////////////////////  
    //  kaos.core.csi.command.FeedbackReceiver implementation
    /////////////////////////////////////////////////////////////////////// 
   public void  onAccepted(Calendar time, Object acceptInfo, String actionID, String robotID)
   {
        // get responseSender
        if ( _responseSenders.containsKey(actionID) ) {
            ResponseSender sender = (ResponseSender) _responseSenders.get(actionID);
            sender.sendResponse ("S-FEEDBACK\nAction status: onAccepted\nFrom: " + robotID + "\nE-FEEDBACK");
        }
        else {
            System.out.println("XMLToCSILink.onAccepted: Did not find response sender to send an onAccepted from: " + robotID);
        }
    }
   
    public void onRefused(Calendar time, Object reason, String actionID, String robotID)
    {
        // get responseSender
        if ( _responseSenders.containsKey(actionID) ) {
            ResponseSender sender = (ResponseSender) _responseSenders.remove(actionID);
            sender.sendResponse ("S-FEEDBACK\nAction status: onRefused\nFrom: " + robotID + "\nE-FEEDBACK");
        }
        else {
            System.out.println("XMLToCSILink.onRefused: Did not find response sender to send an onRefused from: " + robotID);
        }
   }

   public void onCompleted(Calendar time, Object result, String actionID, String robotID)
   {
        // get responseSender
        if ( _responseSenders.containsKey(actionID) ) {
            ResponseSender sender = (ResponseSender) _responseSenders.remove(actionID);
            sender.sendResponse ("S-FEEDBACK\nAction status: onCompleted\nFrom: " + robotID + "\nE-FEEDBACK");
        }
        else {
            System.out.println("XMLToCSILink.onCompleted: Did not find response sender to send an onCompleted from: " + robotID);
        }
   }
   
   public void onStarted(Calendar time, Object startedInfo, String actionID, String robotID)
   {
		// get responseSender
        if ( _responseSenders.containsKey(actionID) ) {
            ResponseSender sender = (ResponseSender) _responseSenders.get(actionID);
            sender.sendResponse ("S-FEEDBACK\nAction status: onStarted\nFrom: " + robotID + "\nE-FEEDBACK");
        }
        else {
            System.out.println("XMLToCSILink.onStarted: Did not find response sender to send a onStarted from: " + robotID);
        }
   }

   public void onAborted(Calendar time, Object reason, String actionID, String robotID)
   {
		// get responseSender
        if ( _responseSenders.containsKey(actionID) ) {
            ResponseSender sender = (ResponseSender) _responseSenders.remove(actionID);
            sender.sendResponse ("S-FEEDBACK\nAction status: onAborted\nFrom: " + robotID + "\nE-FEEDBACK");
        }
        else {
            System.out.println("XMLToCSILink.onAborted: Did not find response sender to send an onAborted from: " + robotID);
        }
   }
   
   public void receiveKAoSStatus(String statusMsg, String actionID, String robotID)
   {
		//_feedback.sendResponse("S-FEEDBACK\n" + statusMsg + "\nE-FEEDBACK");
		System.out.println("XMLToCSILink.receiveKAoSStatus: " + statusMsg + " from: " + robotID);
   }
   
    /////////////////////////////   PRIVATE METHODS BEGIN
    /*
     **/
    private void initialize(String dsTransport)
    throws Exception
    {
        try {
            CSIFactory.setCurrentTransport(dsTransport);
            _registration = CSIFactory.getRegistration();
            _requestMgr = CSIFactory.getRequestManager();
        }
        catch(Exception e){
            e.printStackTrace();
            throw new Exception("  **** Unable to obtain CSI services: Registration for transport: "
                                + dsTransport + ". Exiting...");
        }
        _transportName = dsTransport;
        _domains = new Vector();
        _domains.add(DOMAIN_FOR_ALL_TCP_BASED_PROTOCOL);
        _responseSenders = new Hashtable();
    }
    
    private void initializePolicyChecking()
    {
        try {
            _policyChecking = CSIFactory.getPolicyChecking();
            _policyCheckingInitialized = true;
        }
        catch(Exception e){
            e.printStackTrace();
            System.out.println("\n\n  **** XMLToCSILink: Unable to obtain CSI PolicyChecking! ");
            _policyCheckingInitialized = false;
        }    
    }
    
    /*
        Register domain for all entities communicating with KAoS via direct TCP (vs KAoS-wrapped TCP).
     **/
    public void registerTCPDomain()
        throws DirectoryFailure
    {
        try {
            _registration.registerDomain(DOMAIN_FOR_ALL_TCP_BASED_PROTOCOL);
            _tcpDomainRegistered = true;
        }
        catch(DirectoryFailure de) {
            throw de;
        }
        catch(AlreadyRegisteredException ae) {
            if (_logger.isEnabledFor(Logger.INFO))
                _logger.logMessage(" The following domain has already been registered: " + DOMAIN_FOR_ALL_TCP_BASED_PROTOCOL,Logger.INFO);
        }        
    }
    
public  String getPoseAsString(Pose pose, String receiver)
{
    StringBuffer buffer = new StringBuffer();
    buffer.append(XMLToCSIConstants.GET_POSE);
    buffer.append("\n");
    buffer.append("From: " + receiver);
    buffer.append("\n");
    buffer.append("x: ");
    buffer.append(pose.getX());
    buffer.append("\n");
    buffer.append("y: ");
    buffer.append(pose.getY());
    buffer.append("\n");
    buffer.append("z: ");
    buffer.append(pose.getZ());
    buffer.append("\n");
    buffer.append("roll: ");
    buffer.append(pose.getRoll());
    buffer.append("\n");
    buffer.append("pitch: ");
    buffer.append(pose.getPitch());
    buffer.append("\n");
    buffer.append("yaw: ");
    buffer.append(pose.getYaw());
    return buffer.toString();
}
    
    //
    // ENTRY POINT
    //

    /** Main program entry point. */
    public static void main(String argv[])
    throws Exception 
    {
        // is there anything to do?
        if (argv.length < 2) {
            System.exit(1);
        }
        String fileName = argv[0];
        String transport = argv[1];
        
        System.out.println("\n  <> File to process: " + fileName
                         + "\n  <> Transport:       " + transport);
       
       
        XMLToCSILink link = XMLToCSILink.getInstance(transport);
        /*
        Hashtable registrations = XMLToCSITranslator.translateActionsFromXMLFile(fileName);
        if ( registrations != null ) {
            link.requestAction(null, registrations);
        }
        
       */
        synchronized ( Thread.currentThread() ) {
            Thread.currentThread().wait();
        }

    } // main(String[])

  
} // class XMLToCSILink
